@@ -1,8 +1,8 @@
-from sqlalchemy.ext.declarative import DeclarativeMeta
+from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm import mapper, class_mapper, attributes, object_mapper
 from sqlalchemy.orm.exc import UnmappedClassError, UnmappedColumnError
 from sqlalchemy import Table, Column, ForeignKeyConstraint, Integer
-from sqlalchemy.orm.interfaces import SessionExtension
+from sqlalchemy import event
 from sqlalchemy.orm.properties import RelationshipProperty
 
 def col_references_table(col, table):
@@ -44,9 +44,9 @@ def _history_mapper(local_mapper):
 
         if super_mapper:
             super_fks.append(('version', super_history_mapper.base_mapper.local_table.c.version))
-            cols.append(Column('version', Integer, primary_key=True))
+            cols.append(Column('version', Integer, primary_key=True, autoincrement=False))
         else:
-            cols.append(Column('version', Integer, primary_key=True))
+            cols.append(Column('version', Integer, primary_key=True, autoincrement=False))
 
         if super_fks:
             cols.append(ForeignKeyConstraint(*zip(*super_fks)))
@@ -71,27 +71,29 @@ def _history_mapper(local_mapper):
     versioned_cls = type.__new__(type, "%sHistory" % cls.__name__, bases, {})
 
     m = mapper(
-            versioned_cls, 
-            table, 
-            inherits=super_history_mapper, 
+            versioned_cls,
+            table,
+            inherits=super_history_mapper,
             polymorphic_on=polymorphic_on,
             polymorphic_identity=local_mapper.polymorphic_identity
             )
     cls.__history_mapper__ = m
 
     if not super_history_mapper:
-        cls.version = Column('version', Integer, default=1, nullable=False)
+        local_mapper.local_table.append_column(
+            Column('version', Integer, default=1, nullable=False)
+        )
+        local_mapper.add_property("version", local_mapper.local_table.c.version)
 
 
-class VersionedMeta(DeclarativeMeta):
-    def __init__(cls, classname, bases, dict_):
-        DeclarativeMeta.__init__(cls, classname, bases, dict_)
-
-        try:
-            mapper = class_mapper(cls)
-            _history_mapper(mapper)
-        except UnmappedClassError:
-            pass
+class Versioned(object):
+    @declared_attr
+    def __mapper_cls__(cls):
+        def map(cls, *arg, **kw):
+            mp = mapper(cls, *arg, **kw)
+            _history_mapper(mp)
+            return mp
+        return map
 
 
 def versioned_objects(iter):
@@ -127,9 +129,9 @@ def create_version(obj, session, deleted = False):
             try:
                 prop = obj_mapper.get_property_by_column(obj_col)
             except UnmappedColumnError:
-                # in the case of single table inheritance, there may be 
+                # in the case of single table inheritance, there may be
                 # columns on the mapped table intended for the subclass only.
-                # the "unmapped" status of the subclass column on the 
+                # the "unmapped" status of the subclass column on the
                 # base class is a feature of the declarative module as of sqla 0.5.2.
                 continue
 
@@ -156,8 +158,12 @@ def create_version(obj, session, deleted = False):
         for prop in obj_mapper.iterate_properties:
             if isinstance(prop, RelationshipProperty) and \
                 attributes.get_history(obj, prop.key).has_changes():
-                obj_changed = True
-                break
+                for p in prop.local_columns:
+                    if p.foreign_keys:
+                        obj_changed = True
+                        break
+                if obj_changed is True:
+                    break
 
     if not obj_changed and not deleted:
         return
@@ -169,9 +175,10 @@ def create_version(obj, session, deleted = False):
     session.add(hist)
     obj.version += 1
 
-class VersionedListener(SessionExtension):
-    def before_flush(self, session, flush_context, instances):
+def versioned_session(session):
+    @event.listens_for(session, 'before_flush')
+    def before_flush(session, flush_context, instances):
         for obj in versioned_objects(session.dirty):
             create_version(obj, session)
         for obj in versioned_objects(session.deleted):
-            create_version(obj, session, deleted = True)
\ No newline at end of file
+            create_version(obj, session, deleted = True)
