@@ -1,39 +1,40 @@
 from unittest import TestCase
 from sqlalchemy.ext.declarative import declarative_base
-from history_meta import VersionedMeta, VersionedListener
+from history_meta import Versioned, versioned_session
 from sqlalchemy import create_engine, Column, Integer, String, ForeignKey
-from sqlalchemy.orm import clear_mappers, sessionmaker, deferred, relationship
+from sqlalchemy.orm import clear_mappers, Session, deferred, relationship
 from _lib import ComparableEntity, eq_
 
+engine = None
+
+
 def setup():
     global engine
     engine = create_engine('sqlite://', echo=True)
 
 class TestVersioning(TestCase):
     def setUp(self):
-        global Base, Session, Versioned
-        Base = declarative_base(bind=engine)
-        class Versioned(object):
-            __metaclass__ = VersionedMeta
-            _decl_class_registry = Base._decl_class_registry
-        Session = sessionmaker(extension=VersionedListener())
+        self.session = Session(engine)
+        self.Base = declarative_base()
+        versioned_session(self.session)
 
     def tearDown(self):
+        self.session.close()
         clear_mappers()
-        Base.metadata.drop_all()
+        self.Base.metadata.drop_all(engine)
 
     def create_tables(self):
-        Base.metadata.create_all()
+        self.Base.metadata.create_all(engine)
 
     def test_plain(self):
-        class SomeClass(Versioned, Base, ComparableEntity):
+        class SomeClass(Versioned, self.Base, ComparableEntity):
             __tablename__ = 'sometable'
 
             id = Column(Integer, primary_key=True)
             name = Column(String(50))
 
         self.create_tables()
-        sess = Session()
+        sess = self.session
         sc = SomeClass(name='sc1')
         sess.add(sc)
         sess.commit()
@@ -90,14 +91,14 @@ class TestVersioning(TestCase):
         )
 
     def test_from_null(self):
-        class SomeClass(Versioned, Base, ComparableEntity):
+        class SomeClass(Versioned, self.Base, ComparableEntity):
             __tablename__ = 'sometable'
 
             id = Column(Integer, primary_key=True)
             name = Column(String(50))
 
         self.create_tables()
-        sess = Session()
+        sess = self.session
         sc = SomeClass()
         sess.add(sc)
         sess.commit()
@@ -110,7 +111,7 @@ class TestVersioning(TestCase):
     def test_deferred(self):
         """test versioning of unloaded, deferred columns."""
 
-        class SomeClass(Versioned, Base, ComparableEntity):
+        class SomeClass(Versioned, self.Base, ComparableEntity):
             __tablename__ = 'sometable'
 
             id = Column(Integer, primary_key=True)
@@ -118,7 +119,7 @@ class TestVersioning(TestCase):
             data = deferred(Column(String(25)))
 
         self.create_tables()
-        sess = Session()
+        sess = self.session
         sc = SomeClass(name='sc1', data='somedata')
         sess.add(sc)
         sess.commit()
@@ -141,7 +142,7 @@ class TestVersioning(TestCase):
 
 
     def test_joined_inheritance(self):
-        class BaseClass(Versioned, Base, ComparableEntity):
+        class BaseClass(Versioned, self.Base, ComparableEntity):
             __tablename__ = 'basetable'
 
             id = Column(Integer, primary_key=True)
@@ -168,7 +169,7 @@ class TestVersioning(TestCase):
             __mapper_args__ = {'polymorphic_identity':'same'}
 
         self.create_tables()
-        sess = Session()
+        sess = self.session
 
         sep1 = SubClassSeparatePk(name='sep1', subdata1='sep1subdata')
         base1 = BaseClass(name='base1')
@@ -187,8 +188,8 @@ class TestVersioning(TestCase):
         eq_(
             sess.query(BaseClassHistory).order_by(BaseClassHistory.id).all(),
             [
-                SubClassSeparatePkHistory(id=1, name=u'sep1', type=u'sep', version=1), 
-                BaseClassHistory(id=2, name=u'base1', type=u'base', version=1), 
+                SubClassSeparatePkHistory(id=1, name=u'sep1', type=u'sep', version=1),
+                BaseClassHistory(id=2, name=u'base1', type=u'base', version=1),
                 SubClassSamePkHistory(id=3, name=u'same1', type=u'same', version=1)
             ]
         )
@@ -198,9 +199,9 @@ class TestVersioning(TestCase):
         eq_(
             sess.query(BaseClassHistory).order_by(BaseClassHistory.id, BaseClassHistory.version).all(),
             [
-                SubClassSeparatePkHistory(id=1, name=u'sep1', type=u'sep', version=1), 
-                BaseClassHistory(id=2, name=u'base1', type=u'base', version=1), 
-                SubClassSamePkHistory(id=3, name=u'same1', type=u'same', version=1), 
+                SubClassSeparatePkHistory(id=1, name=u'sep1', type=u'sep', version=1),
+                BaseClassHistory(id=2, name=u'base1', type=u'base', version=1),
+                SubClassSamePkHistory(id=3, name=u'same1', type=u'same', version=1),
                 SubClassSamePkHistory(id=3, name=u'same1', type=u'same', version=2)
             ]
         )
@@ -209,16 +210,16 @@ class TestVersioning(TestCase):
         eq_(
             sess.query(BaseClassHistory).order_by(BaseClassHistory.id, BaseClassHistory.version).all(),
             [
-                SubClassSeparatePkHistory(id=1, name=u'sep1', type=u'sep', version=1), 
-                BaseClassHistory(id=2, name=u'base1', type=u'base', version=1), 
-                BaseClassHistory(id=2, name=u'base1mod', type=u'base', version=2), 
-                SubClassSamePkHistory(id=3, name=u'same1', type=u'same', version=1), 
+                SubClassSeparatePkHistory(id=1, name=u'sep1', type=u'sep', version=1),
+                BaseClassHistory(id=2, name=u'base1', type=u'base', version=1),
+                BaseClassHistory(id=2, name=u'base1mod', type=u'base', version=2),
+                SubClassSamePkHistory(id=3, name=u'same1', type=u'same', version=1),
                 SubClassSamePkHistory(id=3, name=u'same1', type=u'same', version=2)
             ]
         )
 
     def test_single_inheritance(self):
-        class BaseClass(Versioned, Base, ComparableEntity):
+        class BaseClass(Versioned, self.Base, ComparableEntity):
             __tablename__ = 'basetable'
 
             id = Column(Integer, primary_key=True)
@@ -232,7 +233,7 @@ class TestVersioning(TestCase):
             __mapper_args__ = {'polymorphic_identity':'sub'}
 
         self.create_tables()
-        sess = Session()
+        sess = self.session
 
         b1 = BaseClass(name='b1')
         sc = SubClass(name='s1', subname='sc1')
@@ -269,7 +270,7 @@ class TestVersioning(TestCase):
         sess.flush()
 
     def test_unique(self):
-        class SomeClass(Versioned, Base, ComparableEntity):
+        class SomeClass(Versioned, self.Base, ComparableEntity):
             __tablename__ = 'sometable'
 
             id = Column(Integer, primary_key=True)
@@ -277,7 +278,7 @@ class TestVersioning(TestCase):
             data = Column(String(50))
 
         self.create_tables()
-        sess = Session()
+        sess = self.session
         sc = SomeClass(name='sc1', data='sc1')
         sess.add(sc)
         sess.commit()
@@ -294,23 +295,23 @@ class TestVersioning(TestCase):
 
     def test_relationship(self):
 
-        class SomeRelated(Base, ComparableEntity):
+        class SomeRelated(self.Base, ComparableEntity):
             __tablename__ = 'somerelated'
 
             id = Column(Integer, primary_key=True)
 
-        class SomeClass(Versioned, Base, ComparableEntity):
+        class SomeClass(Versioned, self.Base, ComparableEntity):
             __tablename__ = 'sometable'
 
             id = Column(Integer, primary_key=True)
             name = Column(String(50))
             related_id = Column(Integer, ForeignKey('somerelated.id'))
-            related = relationship("SomeRelated")
+            related = relationship("SomeRelated", backref='classes')
 
         SomeClassHistory = SomeClass.__history_mapper__.class_
 
         self.create_tables()
-        sess = Session()
+        sess = self.session
         sc = SomeClass(name='sc1')
         sess.add(sc)
         sess.commit()
@@ -340,3 +341,41 @@ class TestVersioning(TestCase):
 
         assert sc.version == 3
 
+    def test_backref_relationship(self):
+
+        class SomeRelated(self.Base, ComparableEntity):
+            __tablename__ = 'somerelated'
+
+            id = Column(Integer, primary_key=True)
+            name = Column(String(50))
+            related_id = Column(Integer, ForeignKey('sometable.id'))
+            related = relationship("SomeClass", backref='related')
+
+        class SomeClass(Versioned, self.Base, ComparableEntity):
+            __tablename__ = 'sometable'
+
+            id = Column(Integer, primary_key=True)
+
+        self.create_tables()
+        sess = self.session
+        sc = SomeClass()
+        sess.add(sc)
+        sess.commit()
+
+        assert sc.version == 1
+
+        sr = SomeRelated(name='sr', related=sc)
+        sess.add(sr)
+        sess.commit()
+
+        assert sc.version == 1
+
+        sr.name = 'sr2'
+        sess.commit()
+
+        assert sc.version == 1
+
+        sess.delete(sr)
+        sess.commit()
+
+        assert sc.version == 1
