@@ -1,18 +1,19 @@
 # connectors/pyodbc.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-from sqlalchemy.connectors import Connector
-from sqlalchemy.util import asbool
+from . import Connector
+from ..util import asbool
 
 import sys
 import re
 import urllib
 
+
 class PyODBCConnector(Connector):
-    driver='pyodbc'
+    driver = 'pyodbc'
 
     supports_sane_multi_rowcount = False
     # PyODBC unicode is broken on UCS-4 builds
@@ -63,17 +64,17 @@ class PyODBCConnector(Connector):
             dsn_connection = 'dsn' in keys or \
                             ('host' in keys and 'database' not in keys)
             if dsn_connection:
-                connectors= ['dsn=%s' % (keys.pop('host', '') or \
+                connectors = ['dsn=%s' % (keys.pop('host', '') or \
                             keys.pop('dsn', ''))]
             else:
                 port = ''
                 if 'port' in keys and not 'port' in query:
                     port = ',%d' % int(keys.pop('port'))
 
-                connectors = ["DRIVER={%s}" % 
+                connectors = ["DRIVER={%s}" %
                                 keys.pop('driver', self.pyodbc_driver_name),
                               'Server=%s%s' % (keys.pop('host', ''), port),
-                              'Database=%s' % keys.pop('database', '') ]
+                              'Database=%s' % keys.pop('database', '')]
 
             user = keys.pop("user", None)
             if user:
@@ -83,15 +84,15 @@ class PyODBCConnector(Connector):
                 connectors.append("Trusted_Connection=Yes")
 
             # if set to 'Yes', the ODBC layer will try to automagically
-            # convert textual data from your database encoding to your 
-            # client encoding.  This should obviously be set to 'No' if 
-            # you query a cp1253 encoded database from a latin1 client... 
+            # convert textual data from your database encoding to your
+            # client encoding.  This should obviously be set to 'No' if
+            # you query a cp1253 encoded database from a latin1 client...
             if 'odbc_autotranslate' in keys:
                 connectors.append("AutoTranslate=%s" %
                                     keys.pop("odbc_autotranslate"))
 
-            connectors.extend(['%s=%s' % (k,v) for k,v in keys.iteritems()])
-        return [[";".join (connectors)], connect_args]
+            connectors.extend(['%s=%s' % (k, v) for k, v in keys.iteritems()])
+        return [[";".join(connectors)], connect_args]
 
     def is_disconnect(self, e, connection, cursor):
         if isinstance(e, self.dbapi.ProgrammingError):
@@ -117,18 +118,20 @@ class PyODBCConnector(Connector):
                             ))
 
         if self.freetds:
-            self.freetds_driver_version = dbapi_con.getinfo(pyodbc.SQL_DRIVER_VER)
+            self.freetds_driver_version = dbapi_con.getinfo(
+                pyodbc.SQL_DRIVER_VER)
 
         # the "Py2K only" part here is theoretical.
         # have not tried pyodbc + python3.1 yet.
         # Py2K
-        self.supports_unicode_statements = not self.freetds and not self.easysoft
+        self.supports_unicode_statements = (
+            not self.freetds and not self.easysoft)
         if self._user_supports_unicode_binds is not None:
             self.supports_unicode_binds = self._user_supports_unicode_binds
         else:
-            self.supports_unicode_binds = (not self.freetds or 
-                                            self.freetds_driver_version >= '0.91'
-                                            ) and not self.easysoft
+            self.supports_unicode_binds = (
+                not self.freetds or self.freetds_driver_version >= '0.91'
+            ) and not self.easysoft
         # end Py2K
 
         # run other initialization which asks for user name, etc.
