@@ -1,17 +1,17 @@
 # mysql/__init__.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-from sqlalchemy.dialects.mysql import base, mysqldb, oursql, \
+from . import base, mysqldb, oursql, \
                                 pyodbc, zxjdbc, mysqlconnector, pymysql,\
-                                gaerdbms
+                                gaerdbms, cymysql
 
 # default dialect
 base.dialect = mysqldb.dialect
 
-from sqlalchemy.dialects.mysql.base import \
+from .base import \
     BIGINT, BINARY, BIT, BLOB, BOOLEAN, CHAR, DATE, DATETIME, \
     DECIMAL, DOUBLE, ENUM, DECIMAL,\
     FLOAT, INTEGER, INTEGER, LONGBLOB, LONGTEXT, MEDIUMBLOB, \
