@@ -1,17 +1,15 @@
 # sybase/pysybase.py
-# Copyright (C) 2010-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2010-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 """
-Support for Sybase via the python-sybase driver.
-
-http://python-sybase.sourceforge.net/
-
-Connect strings are of the form::
-
-    sybase+pysybase://<username>:<password>@<dsn>/[database name]
+.. dialect:: sybase+pysybase
+    :name: Python-Sybase
+    :dbapi: Sybase
+    :connectstring: sybase+pysybase://<username>:<password>@<dsn>/[database name]
+    :url: http://python-sybase.sourceforge.net/
 
 Unicode Support
 ---------------
@@ -33,6 +31,7 @@ class _SybNumeric(sqltypes.Numeric):
         else:
             return sqltypes.Numeric.result_processor(self, dialect, type_)
 
+
 class SybaseExecutionContext_pysybase(SybaseExecutionContext):
 
     def set_ddl_autocommit(self, dbapi_connection, value):
@@ -55,14 +54,15 @@ class SybaseSQLCompiler_pysybase(SybaseSQLCompiler):
     def bindparam_string(self, name, **kw):
         return "@" + name
 
+
 class SybaseDialect_pysybase(SybaseDialect):
     driver = 'pysybase'
     execution_ctx_cls = SybaseExecutionContext_pysybase
     statement_compiler = SybaseSQLCompiler_pysybase
 
-    colspecs={
-       sqltypes.Numeric:_SybNumeric,
-       sqltypes.Float:sqltypes.Float
+    colspecs = {
+       sqltypes.Numeric: _SybNumeric,
+       sqltypes.Float: sqltypes.Float
     }
 
     @classmethod
