@@ -1,5 +1,5 @@
 # ext/associationproxy.py
-# Copyright (C) 2005-2012 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -15,38 +15,37 @@ See the example ``examples/association/proxied_association.py``.
 import itertools
 import operator
 import weakref
-from sqlalchemy import exceptions
-from sqlalchemy import orm
-from sqlalchemy import util
-from sqlalchemy.orm import collections, ColumnProperty
-from sqlalchemy.sql import not_
+from .. import exc, orm, util
+from ..orm import collections, interfaces
+from ..sql import not_
 
 
 def association_proxy(target_collection, attr, **kw):
     """Return a Python property implementing a view of a target
-    attribute which references an attribute on members of the 
+    attribute which references an attribute on members of the
     target.
-    
-    The returned value is an instance of :class:`.AssociationProxy`.
-    
-    Implements a Python property representing a relationship as a collection of
-    simpler values, or a scalar value.  The proxied property will mimic the collection type of
-    the target (list, dict or set), or, in the case of a one to one relationship,
-    a simple scalar value.
 
-    :param target_collection: Name of the attribute we'll proxy to.  
+    The returned value is an instance of :class:`.AssociationProxy`.
+
+    Implements a Python property representing a relationship as a collection
+    of simpler values, or a scalar value.  The proxied property will mimic
+    the collection type of the target (list, dict or set), or, in the case of
+    a one to one relationship, a simple scalar value.
+
+    :param target_collection: Name of the attribute we'll proxy to.
       This attribute is typically mapped by
       :func:`~sqlalchemy.orm.relationship` to link to a target collection, but
       can also be a many-to-one or non-scalar relationship.
 
-    :param attr: Attribute on the associated instance or instances we'll proxy for.
+    :param attr: Attribute on the associated instance or instances we'll
+      proxy for.
 
       For example, given a target collection of [obj1, obj2], a list created
       by this proxy property would look like [getattr(obj1, *attr*),
       getattr(obj2, *attr*)]
 
-      If the relationship is one-to-one or otherwise uselist=False, then simply:
-      getattr(obj, *attr*)
+      If the relationship is one-to-one or otherwise uselist=False, then
+      simply: getattr(obj, *attr*)
 
     :param creator: optional.
 
@@ -76,51 +75,66 @@ def association_proxy(target_collection, attr, **kw):
     return AssociationProxy(target_collection, attr, **kw)
 
 
-class AssociationProxy(object):
+ASSOCIATION_PROXY = util.symbol('ASSOCIATION_PROXY')
+"""Symbol indicating an :class:`_InspectionAttr` that's
+    of type :class:`.AssociationProxy`.
+
+   Is assigned to the :attr:`._InspectionAttr.extension_type`
+   attibute.
+
+"""
+
+class AssociationProxy(interfaces._InspectionAttr):
     """A descriptor that presents a read/write view of an object attribute."""
 
+    is_attribute = False
+    extension_type = ASSOCIATION_PROXY
+
+
     def __init__(self, target_collection, attr, creator=None,
-                 getset_factory=None, proxy_factory=None, 
+                 getset_factory=None, proxy_factory=None,
                  proxy_bulk_set=None):
         """Construct a new :class:`.AssociationProxy`.
-        
+
         The :func:`.association_proxy` function is provided as the usual
         entrypoint here, though :class:`.AssociationProxy` can be instantiated
         and/or subclassed directly.
 
-        :param target_collection: Name of the collection we'll proxy to, 
+        :param target_collection: Name of the collection we'll proxy to,
           usually created with :func:`.relationship`.
 
-        :param attr: Attribute on the collected instances we'll proxy for.  For example,
-          given a target collection of [obj1, obj2], a list created by this
-          proxy property would look like [getattr(obj1, attr), getattr(obj2,
-          attr)]
+        :param attr: Attribute on the collected instances we'll proxy
+          for.  For example, given a target collection of [obj1, obj2], a
+          list created by this proxy property would look like
+          [getattr(obj1, attr), getattr(obj2, attr)]
 
-        :param creator: Optional. When new items are added to this proxied collection, new
-          instances of the class collected by the target collection will be
-          created.  For list and set collections, the target class constructor
-          will be called with the 'value' for the new instance.  For dict
-          types, two arguments are passed: key and value.
+        :param creator: Optional. When new items are added to this proxied
+          collection, new instances of the class collected by the target
+          collection will be created.  For list and set collections, the
+          target class constructor will be called with the 'value' for the
+          new instance.  For dict types, two arguments are passed:
+          key and value.
 
           If you want to construct instances differently, supply a 'creator'
           function that takes arguments as above and returns instances.
 
-        :param getset_factory: Optional.  Proxied attribute access is automatically handled by
-          routines that get and set values based on the `attr` argument for
-          this proxy.
+        :param getset_factory: Optional.  Proxied attribute access is
+          automatically handled by routines that get and set values based on
+          the `attr` argument for this proxy.
 
           If you would like to customize this behavior, you may supply a
           `getset_factory` callable that produces a tuple of `getter` and
           `setter` functions.  The factory is called with two arguments, the
           abstract type of the underlying collection and this proxy instance.
 
-        :param proxy_factory: Optional.  The type of collection to emulate is determined by
-          sniffing the target collection.  If your collection type can't be
-          determined by duck typing or you'd like to use a different
-          collection implementation, you may supply a factory function to
-          produce those collections.  Only applicable to non-scalar relationships.
+        :param proxy_factory: Optional.  The type of collection to emulate is
+          determined by sniffing the target collection.  If your collection
+          type can't be determined by duck typing or you'd like to use a
+          different collection implementation, you may supply a factory
+          function to produce those collections.  Only applicable to
+          non-scalar relationships.
 
-        :param proxy_bulk_set: Optional, use with proxy_factory.  See 
+        :param proxy_bulk_set: Optional, use with proxy_factory.  See
           the _set() method for details.
 
         """
@@ -140,11 +154,11 @@ class AssociationProxy(object):
     def remote_attr(self):
         """The 'remote' :class:`.MapperProperty` referenced by this
         :class:`.AssociationProxy`.
-        
+
         .. versionadded:: 0.7.3
-        
+
         See also:
-        
+
         :attr:`.AssociationProxy.attr`
 
         :attr:`.AssociationProxy.local_attr`
@@ -158,9 +172,9 @@ class AssociationProxy(object):
         :class:`.AssociationProxy`.
 
         .. versionadded:: 0.7.3
-        
+
         See also:
-        
+
         :attr:`.AssociationProxy.attr`
 
         :attr:`.AssociationProxy.remote_attr`
@@ -171,20 +185,20 @@ class AssociationProxy(object):
     @property
     def attr(self):
         """Return a tuple of ``(local_attr, remote_attr)``.
-        
-        This attribute is convenient when specifying a join 
+
+        This attribute is convenient when specifying a join
         using :meth:`.Query.join` across two relationships::
-        
+
             sess.query(Parent).join(*Parent.proxied.attr)
 
         .. versionadded:: 0.7.3
-        
+
         See also:
-        
+
         :attr:`.AssociationProxy.local_attr`
 
         :attr:`.AssociationProxy.remote_attr`
-        
+
         """
         return (self.local_attr, self.remote_attr)
 
@@ -195,10 +209,10 @@ class AssociationProxy(object):
     @util.memoized_property
     def target_class(self):
         """The intermediary class handled by this :class:`.AssociationProxy`.
-        
+
         Intercepted append/set/assignment events will result
         in the generation of new instances of this class.
-        
+
         """
         return self._get_property().mapper.class_
 
@@ -281,7 +295,8 @@ class AssociationProxy(object):
         self.collection_class = util.duck_type_collection(lazy_collection())
 
         if self.proxy_factory:
-            return self.proxy_factory(lazy_collection, creator, self.value_attr, self)
+            return self.proxy_factory(
+                lazy_collection, creator, self.value_attr, self)
 
         if self.getset_factory:
             getter, setter = self.getset_factory(self.collection_class, self)
@@ -289,13 +304,16 @@ class AssociationProxy(object):
             getter, setter = self._default_getset(self.collection_class)
 
         if self.collection_class is list:
-            return _AssociationList(lazy_collection, creator, getter, setter, self)
+            return _AssociationList(
+                lazy_collection, creator, getter, setter, self)
         elif self.collection_class is dict:
-            return _AssociationDict(lazy_collection, creator, getter, setter, self)
+            return _AssociationDict(
+                lazy_collection, creator, getter, setter, self)
         elif self.collection_class is set:
-            return _AssociationSet(lazy_collection, creator, getter, setter, self)
+            return _AssociationSet(
+                lazy_collection, creator, getter, setter, self)
         else:
-            raise exceptions.ArgumentError(
+            raise exc.ArgumentError(
                 'could not guess which interface to use for '
                 'collection_class "%s" backing "%s"; specify a '
                 'proxy_factory and proxy_bulk_set manually' %
@@ -323,7 +341,7 @@ class AssociationProxy(object):
         elif self.collection_class is set:
             proxy.update(values)
         else:
-            raise exceptions.ArgumentError(
+            raise exc.ArgumentError(
                'no proxy_bulk_set supplied for custom '
                'collection_class implementation')
 
@@ -333,18 +351,20 @@ class AssociationProxy(object):
 
     def any(self, criterion=None, **kwargs):
         """Produce a proxied 'any' expression using EXISTS.
-        
+
         This expression will be a composed product
         using the :meth:`.RelationshipProperty.Comparator.any`
-        and/or :meth:`.RelationshipProperty.Comparator.has` 
+        and/or :meth:`.RelationshipProperty.Comparator.has`
         operators of the underlying proxied attributes.
 
         """
 
         if self._value_is_scalar:
-            value_expr = getattr(self.target_class, self.value_attr).has(criterion, **kwargs)
+            value_expr = getattr(
+                self.target_class, self.value_attr).has(criterion, **kwargs)
         else:
-            value_expr = getattr(self.target_class, self.value_attr).any(criterion, **kwargs)
+            value_expr = getattr(
+                self.target_class, self.value_attr).any(criterion, **kwargs)
 
         # check _value_is_scalar here, otherwise
         # we're scalar->scalar - call .any() so that
@@ -360,12 +380,12 @@ class AssociationProxy(object):
 
     def has(self, criterion=None, **kwargs):
         """Produce a proxied 'has' expression using EXISTS.
-        
+
         This expression will be a composed product
         using the :meth:`.RelationshipProperty.Comparator.any`
-        and/or :meth:`.RelationshipProperty.Comparator.has` 
+        and/or :meth:`.RelationshipProperty.Comparator.has`
         operators of the underlying proxied attributes.
-        
+
         """
 
         return self._comparator.has(
@@ -375,7 +395,7 @@ class AssociationProxy(object):
 
     def contains(self, obj):
         """Produce a proxied 'contains' expression using EXISTS.
-        
+
         This expression will be a composed product
         using the :meth:`.RelationshipProperty.Comparator.any`
         , :meth:`.RelationshipProperty.Comparator.has`,
@@ -405,18 +425,19 @@ class _lazy_collection(object):
     def __call__(self):
         obj = self.ref()
         if obj is None:
-            raise exceptions.InvalidRequestError(
+            raise exc.InvalidRequestError(
                "stale association proxy, parent object has gone out of "
                "scope")
         return getattr(obj, self.target)
 
     def __getstate__(self):
-        return {'obj':self.ref(), 'target':self.target}
+        return {'obj': self.ref(), 'target': self.target}
 
     def __setstate__(self, state):
         self.ref = weakref.ref(state['obj'])
         self.target = state['target']
 
+
 class _AssociationCollection(object):
     def __init__(self, lazy_collection, creator, getter, setter, parent):
         """Constructs an _AssociationCollection.
@@ -458,13 +479,14 @@ class _AssociationCollection(object):
         return bool(self.col)
 
     def __getstate__(self):
-        return {'parent':self.parent, 'lazy_collection':self.lazy_collection}
+        return {'parent': self.parent, 'lazy_collection': self.lazy_collection}
 
     def __setstate__(self, state):
         self.parent = state['parent']
         self.lazy_collection = state['lazy_collection']
         self.parent._inflate(self)
 
+
 class _AssociationList(_AssociationCollection):
     """Generic, converting, list-to-list proxy."""
 
@@ -654,6 +676,8 @@ class _AssociationList(_AssociationCollection):
 
 
 _NotProvided = util.symbol('_NotProvided')
+
+
 class _AssociationDict(_AssociationCollection):
     """Generic, converting, dict-to-dict proxy."""
 
@@ -736,7 +760,7 @@ class _AssociationDict(_AssociationCollection):
         return self.col.iterkeys()
 
     def values(self):
-        return [ self._get(member) for member in self.col.values() ]
+        return [self._get(member) for member in self.col.values()]
 
     def itervalues(self):
         for key in self.col:
@@ -768,8 +792,8 @@ class _AssociationDict(_AssociationCollection):
                             len(a))
         elif len(a) == 1:
             seq_or_map = a[0]
-            # discern dict from sequence - took the advice
-            # from http://www.voidspace.org.uk/python/articles/duck_typing.shtml
+            # discern dict from sequence - took the advice from
+            # http://www.voidspace.org.uk/python/articles/duck_typing.shtml
             # still not perfect :(
             if hasattr(seq_or_map, 'keys'):
                 for item in seq_or_map:
