@@ -1,24 +1,23 @@
-from test.lib.testing import eq_, assert_raises, \
+from sqlalchemy.testing import eq_, assert_raises, \
     assert_raises_message
 from sqlalchemy import exc as sa_exc, util, Integer, String, ForeignKey
 from sqlalchemy.orm import exc as orm_exc, mapper, relationship, \
-    sessionmaker, Session
-from test.lib import testing, profiling
-from test.lib import fixtures
-from test.lib.schema import Table, Column
+    sessionmaker, Session, defer
+from sqlalchemy import testing
+from sqlalchemy.testing import profiling
+from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.schema import Table, Column
 import sys
 
 class MergeTest(fixtures.MappedTest):
 
-    __requires__ = 'cpython',
-
     @classmethod
     def define_tables(cls, metadata):
-        parent = Table('parent', metadata, Column('id', Integer,
+        Table('parent', metadata, Column('id', Integer,
                        primary_key=True,
                        test_needs_autoincrement=True), Column('data',
                        String(20)))
-        child = Table('child', metadata, Column('id', Integer,
+        Table('child', metadata, Column('id', Integer,
                       primary_key=True, test_needs_autoincrement=True),
                       Column('data', String(20)), Column('parent_id',
                       Integer, ForeignKey('parent.id'), nullable=False))
@@ -38,8 +37,8 @@ class MergeTest(fixtures.MappedTest):
                                 cls.tables.parent,
                                 cls.tables.child)
 
-        mapper(Parent, parent, properties={'children'
-               : relationship(Child, backref='parent')})
+        mapper(Parent, parent, properties={'children':
+                        relationship(Child, backref='parent')})
         mapper(Child, child)
 
     @classmethod
@@ -47,8 +46,7 @@ class MergeTest(fixtures.MappedTest):
         parent, child = cls.tables.parent, cls.tables.child
 
         parent.insert().execute({'id': 1, 'data': 'p1'})
-        child.insert().execute({'id': 1, 'data': 'p1c1', 'parent_id'
-                               : 1})
+        child.insert().execute({'id': 1, 'data': 'p1c1', 'parent_id': 1})
 
     def test_merge_no_load(self):
         Parent = self.classes.Parent
@@ -61,21 +59,18 @@ class MergeTest(fixtures.MappedTest):
         # down from 185 on this this is a small slice of a usually
         # bigger operation so using a small variance
 
-        @profiling.function_call_count(variance=0.05,
-                versions={'2.7':80, '2.6':80, '2.5':94, '3': 83})
-        def go():
+        @profiling.function_call_count(variance=0.10)
+        def go1():
             return sess2.merge(p1, load=False)
-        p2 = go()
+        p2 = go1()
 
         # third call, merge object already present. almost no calls.
 
-        @profiling.function_call_count(variance=0.05,
-                versions={'2.7':11, '2.6':11, '2.5':15, '3': 12})
-        def go():
+        @profiling.function_call_count(variance=0.10)
+        def go2():
             return sess2.merge(p2, load=False)
-        p3 = go()
+        go2()
 
-    @testing.only_on('sqlite', 'Call counts tailored to pysqlite')
     def test_merge_load(self):
         Parent = self.classes.Parent
 
@@ -88,48 +83,39 @@ class MergeTest(fixtures.MappedTest):
         # using sqlite3 the C extension took it back up to approx. 1257
         # (py2.6)
 
-        @profiling.function_call_count(variance=0.10,
-                                versions={'2.5':1050, '2.6':1050,
-                                        '2.6+cextension':1005, 
-                                        '2.7':1005,
-                                        '3':1050}
-                            )
+        @profiling.function_call_count()
         def go():
             p2 = sess2.merge(p1)
         go()
 
         # one more time, count the SQL
 
+        def go2():
+            p2 = sess2.merge(p1)
         sess2 = sessionmaker(testing.db)()
-        self.assert_sql_count(testing.db, go, 2)
+        self.assert_sql_count(testing.db, go2, 2)
 
 class LoadManyToOneFromIdentityTest(fixtures.MappedTest):
     """test overhead associated with many-to-one fetches.
 
-    Prior to the refactor of LoadLazyAttribute and 
+    Prior to the refactor of LoadLazyAttribute and
     query._get(), the load from identity map took 2x
     as many calls (65K calls here instead of around 33K)
     to load 1000 related objects from the identity map.
 
     """
 
-    # only need to test for unexpected variance in a large call 
-    # count here,
-    # so remove some platforms that have wildly divergent
-    # callcounts.
-    __requires__ = 'python25', 'cpython'
-    __unsupported_on__ = 'postgresql+pg8000', 'mysql+pymysql'
 
     @classmethod
     def define_tables(cls, metadata):
-        parent = Table('parent', metadata, 
-                        Column('id', Integer, primary_key=True), 
+        Table('parent', metadata,
+                        Column('id', Integer, primary_key=True),
                        Column('data', String(20)),
                        Column('child_id', Integer, ForeignKey('child.id'))
                        )
 
-        child = Table('child', metadata, 
-                    Column('id', Integer,primary_key=True),
+        Table('child', metadata,
+                    Column('id', Integer, primary_key=True),
                   Column('data', String(20))
                  )
 
@@ -162,10 +148,10 @@ class LoadManyToOneFromIdentityTest(fixtures.MappedTest):
         ])
         parent.insert().execute([
             {
-                'id':i, 
-                'data':'p%dc%d' % (i, (i % 250) + 1), 
+                'id':i,
+                'data':'p%dc%d' % (i, (i % 250) + 1),
                 'child_id':(i % 250) + 1
-            } 
+            }
             for i in xrange(1, 1000)
         ])
 
@@ -176,7 +162,7 @@ class LoadManyToOneFromIdentityTest(fixtures.MappedTest):
         parents = sess.query(Parent).all()
 
 
-        @profiling.function_call_count(108019, variance=.2)
+        @profiling.function_call_count(variance=.2)
         def go():
             for p in parents:
                 p.child
@@ -189,30 +175,29 @@ class LoadManyToOneFromIdentityTest(fixtures.MappedTest):
         parents = sess.query(Parent).all()
         children = sess.query(Child).all()
 
-        @profiling.function_call_count(17987, {'3':18987})
+        @profiling.function_call_count()
         def go():
             for p in parents:
                 p.child
         go()
 
 class MergeBackrefsTest(fixtures.MappedTest):
-    __only_on__ = 'sqlite'  # keep things simple
 
     @classmethod
     def define_tables(cls, metadata):
         Table('a', metadata,
-            Column('id', Integer, primary_key=True), 
+            Column('id', Integer, primary_key=True),
             Column('c_id', Integer, ForeignKey('c.id'))
         )
         Table('b', metadata,
-            Column('id', Integer, primary_key=True), 
+            Column('id', Integer, primary_key=True),
             Column('a_id', Integer, ForeignKey('a.id'))
         )
         Table('c', metadata,
-            Column('id', Integer, primary_key=True), 
+            Column('id', Integer, primary_key=True),
         )
         Table('d', metadata,
-            Column('id', Integer, primary_key=True), 
+            Column('id', Integer, primary_key=True),
             Column('a_id', Integer, ForeignKey('a.id'))
         )
 
@@ -231,12 +216,12 @@ class MergeBackrefsTest(fixtures.MappedTest):
     def setup_mappers(cls):
         A, B, C, D = cls.classes.A, cls.classes.B, \
                     cls.classes.C, cls.classes.D
-        a, b, c, d= cls.tables.a, cls.tables.b, \
+        a, b, c, d = cls.tables.a, cls.tables.b, \
                     cls.tables.c, cls.tables.d
         mapper(A, a, properties={
-            'bs':relationship(B, backref='a'),
-            'c':relationship(C, backref='as'),
-            'ds':relationship(D, backref='a'),
+            'bs': relationship(B, backref='a'),
+            'c': relationship(C, backref='as'),
+            'ds': relationship(D, backref='a'),
         })
         mapper(B, b)
         mapper(C, c)
@@ -248,28 +233,80 @@ class MergeBackrefsTest(fixtures.MappedTest):
                     cls.classes.C, cls.classes.D
         s = Session()
         s.add_all([
-            A(id=i, 
-                bs=[B(id=(i * 50) + j) for j in xrange(1, 50)],
+            A(id=i,
+                bs=[B(id=(i * 5) + j) for j in xrange(1, 5)],
                 c=C(id=i),
-                ds=[D(id=(i * 50) + j) for j in xrange(1, 50)]
+                ds=[D(id=(i * 5) + j) for j in xrange(1, 5)]
             )
-            for i in xrange(1, 50)
+            for i in xrange(1, 5)
         ])
         s.commit()
 
-    @profiling.function_call_count(1092497, variance=.10)
+    @profiling.function_call_count(variance=.10)
     def test_merge_pending_with_all_pks(self):
         A, B, C, D = self.classes.A, self.classes.B, \
                     self.classes.C, self.classes.D
         s = Session()
         for a in [
-            A(id=i, 
-                bs=[B(id=(i * 50) + j) for j in xrange(1, 50)],
+            A(id=i,
+                bs=[B(id=(i * 5) + j) for j in xrange(1, 5)],
                 c=C(id=i),
-                ds=[D(id=(i * 50) + j) for j in xrange(1, 50)]
+                ds=[D(id=(i * 5) + j) for j in xrange(1, 5)]
             )
-            for i in xrange(1, 50)
+            for i in xrange(1, 5)
         ]:
             s.merge(a)
 
+class DeferOptionsTest(fixtures.MappedTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('a', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('x', String(5)),
+            Column('y', String(5)),
+            Column('z', String(5)),
+            Column('q', String(5)),
+            Column('p', String(5)),
+            Column('r', String(5)),
+        )
+
+    @classmethod
+    def setup_classes(cls):
+        class A(cls.Basic):
+            pass
+
+    @classmethod
+    def setup_mappers(cls):
+        A = cls.classes.A
+        a = cls.tables.a
+        mapper(A, a)
+
+    @classmethod
+    def insert_data(cls):
+        A = cls.classes.A
+        s = Session()
+        s.add_all([
+            A(id=i,
+                **dict((letter, "%s%d" % (letter, i)) for letter in
+                        ['x', 'y', 'z', 'p', 'q', 'r'])
+            ) for i in range(1, 1001)
+        ])
+        s.commit()
+
+    @profiling.function_call_count(variance=.10)
+    def test_baseline(self):
+        # as of [ticket:2778], this is at 39025
+        A = self.classes.A
+        s = Session()
+        s.query(A).all()
+
+    @profiling.function_call_count(variance=.10)
+    def test_defer_many_cols(self):
+        # with [ticket:2778], this goes from 50805 to 32817,
+        # as it should be fewer function calls than the baseline
+        A = self.classes.A
+        s = Session()
+        s.query(A).options(
+            *[defer(letter) for letter in ['x', 'y', 'z', 'p', 'q', 'r']]).\
+            all()
 
