@@ -8,7 +8,8 @@ import sys
 import time
 from sqlalchemy import *
 from sqlalchemy.orm import *
-from test.lib import *
+from sqlalchemy.testing import fixtures, engines, profiling
+from sqlalchemy import testing
 ITERATIONS = 1
 dbapi_session = engines.ReplayableSession()
 metadata = None
@@ -326,49 +327,36 @@ class ZooMarkTest(fixtures.TestBase):
     def test_profile_0(self):
         global metadata, session
         player = lambda : dbapi_session.player()
-        engine = create_engine('postgresql:///', creator=player)
+        engine = create_engine('postgresql:///', creator=player,
+                    use_native_hstore=False)
         metadata = MetaData(engine)
         session = sessionmaker(engine)()
         engine.connect()
 
-    @profiling.function_call_count(5600)
     def test_profile_1_create_tables(self):
         self.test_baseline_1_create_tables()
 
-    @profiling.function_call_count(5786, {'2.7+cextension':5683, 
-                                            '2.6+cextension':5992})
+    @profiling.function_call_count()
     def test_profile_1a_populate(self):
         self.test_baseline_1a_populate()
 
-    @profiling.function_call_count(393, {'3.2':360})
+    @profiling.function_call_count()
     def test_profile_2_insert(self):
         self.test_baseline_2_insert()
 
-    # this number...
-
-    @profiling.function_call_count(6783, {
-        '2.6': 6058,
-        '2.7': 5922,
-        '2.7+cextension': 5714,
-        '2.6+cextension': 5714,
-        '3.2':5787,
-        })
+    @profiling.function_call_count()
     def test_profile_3_properties(self):
         self.test_baseline_3_properties()
 
-    # and this number go down slightly when using the C extensions
-
-    @profiling.function_call_count(17698, {'2.7+cextension':17698, '2.6': 18943, '2.7':19110})
+    @profiling.function_call_count()
     def test_profile_4_expressions(self):
         self.test_baseline_4_expressions()
 
-    @profiling.function_call_count(1172, {'2.6+cextension': 1090,
-                                   '2.7+cextension': 1086},
-                                   variance=0.1)
+    @profiling.function_call_count()
     def test_profile_5_aggregates(self):
         self.test_baseline_5_aggregates()
 
-    @profiling.function_call_count(2545)
+    @profiling.function_call_count()
     def test_profile_6_editing(self):
         self.test_baseline_6_editing()
 
