@@ -1,15 +1,15 @@
-from sqlalchemy.test.testing import eq_, assert_raises, \
+from sqlalchemy.testing import eq_, assert_raises, \
     assert_raises_message
 from sqlalchemy import *
 from sqlalchemy.orm import *
 from sqlalchemy.orm import exc as orm_exc
-from sqlalchemy.test import *
+from sqlalchemy.testing import *
 import sqlalchemy as sa
-from sqlalchemy.test import testing
-from test.orm import _base
+from sqlalchemy import testing
+from sqlalchemy.testing import fixtures
 from sqlalchemy.orm import attributes
-from sqlalchemy.test.testing import eq_
-from sqlalchemy.test.schema import Table, Column
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing.schema import Table, Column
 
 
 class Employee(object):
@@ -64,7 +64,7 @@ class Company(object):
     pass
 
 
-class ConcreteTest(_base.MappedTest):
+class ConcreteTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
@@ -166,8 +166,18 @@ class ConcreteTest(_base.MappedTest):
                                polymorphic_identity='hacker')
         session = create_session()
         tom = Manager('Tom', 'knows how to manage things')
+
+        assert_raises_message(AttributeError,
+            "does not implement attribute .?'type' at the instance level.",
+            setattr, tom, "type", "sometype")
+
         jerry = Engineer('Jerry', 'knows how to program')
         hacker = Hacker('Kurt', 'Badass', 'knows how to hack')
+
+        assert_raises_message(AttributeError,
+            "does not implement attribute .?'type' at the instance level.",
+            setattr, hacker, "type", "sometype")
+
         session.add_all((tom, jerry, hacker))
         session.flush()
 
@@ -372,7 +382,7 @@ class ConcreteTest(_base.MappedTest):
         self.assert_sql_count(testing.db, go, 1)
 
 
-class PropertyInheritanceTest(_base.MappedTest):
+class PropertyInheritanceTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
@@ -397,7 +407,7 @@ class PropertyInheritanceTest(_base.MappedTest):
     @classmethod
     def setup_classes(cls):
 
-        class A(_base.ComparableEntity):
+        class A(cls.Comparable):
             pass
 
         class B(A):
@@ -406,11 +416,17 @@ class PropertyInheritanceTest(_base.MappedTest):
         class C(A):
             pass
 
-        class Dest(_base.ComparableEntity):
+        class Dest(cls.Comparable):
             pass
 
-    @testing.resolve_artifact_names
     def test_noninherited_warning(self):
+        A, B, b_table, a_table, Dest, dest_table = (self.classes.A,
+                                self.classes.B,
+                                self.tables.b_table,
+                                self.tables.a_table,
+                                self.classes.Dest,
+                                self.tables.dest_table)
+
         mapper(A, a_table, properties={'some_dest': relationship(Dest)})
         mapper(B, b_table, inherits=A, concrete=True)
         mapper(Dest, dest_table)
@@ -430,8 +446,14 @@ class PropertyInheritanceTest(_base.MappedTest):
         mapper(B, b_table, inherits=A, concrete=True)
         mapper(Dest, dest_table)
 
-    @testing.resolve_artifact_names
     def test_inheriting(self):
+        A, B, b_table, a_table, Dest, dest_table = (self.classes.A,
+                                self.classes.B,
+                                self.tables.b_table,
+                                self.tables.a_table,
+                                self.classes.Dest,
+                                self.tables.dest_table)
+
         mapper(A, a_table, properties={
                 'some_dest': relationship(Dest,back_populates='many_a')
             })
@@ -441,7 +463,7 @@ class PropertyInheritanceTest(_base.MappedTest):
                 })
 
         mapper(Dest, dest_table, properties={
-                    'many_a': relationship(A,back_populates='some_dest'), 
+                    'many_a': relationship(A,back_populates='some_dest'),
                     'many_b': relationship(B,back_populates='some_dest')
                 })
         sess = sessionmaker()()
@@ -464,12 +486,21 @@ class PropertyInheritanceTest(_base.MappedTest):
         assert dest1.many_b == [b1, b2]
         assert sess.query(B).filter(B.bname == 'b1').one() is b1
 
-    @testing.resolve_artifact_names
     def test_polymorphic_backref(self):
         """test multiple backrefs to the same polymorphically-loading
         attribute."""
 
-        ajoin = polymorphic_union({'a': a_table, 'b': b_table, 'c':c_table}, 
+        A, C, B, c_table, b_table, a_table, Dest, dest_table = (self.classes.A,
+                                self.classes.C,
+                                self.classes.B,
+                                self.tables.c_table,
+                                self.tables.b_table,
+                                self.tables.a_table,
+                                self.classes.Dest,
+                                self.tables.dest_table)
+
+
+        ajoin = polymorphic_union({'a': a_table, 'b': b_table, 'c':c_table},
                                 'type','ajoin')
         mapper(
             A,
@@ -503,7 +534,7 @@ class PropertyInheritanceTest(_base.MappedTest):
 
         mapper(Dest, dest_table, properties={
                 'many_a': relationship(A,
-                            back_populates='some_dest', 
+                            back_populates='some_dest',
                             order_by=ajoin.c.id)
                         }
                 )
@@ -534,18 +565,26 @@ class PropertyInheritanceTest(_base.MappedTest):
         def go():
             eq_(
                 [
-                    Dest(many_a=[A(aname='a1'), 
-                                    B(bname='b1'), 
+                    Dest(many_a=[A(aname='a1'),
+                                    B(bname='b1'),
                                     B(bname='b2'),
-                                    C(cname='c1')]), 
+                                    C(cname='c1')]),
                     Dest(many_a=[A(aname='a2'), C(cname='c2')])],
                 sess.query(Dest).options(joinedload(Dest.many_a)).order_by(Dest.id).all())
 
         self.assert_sql_count(testing.db, go, 1)
 
-    @testing.resolve_artifact_names
     def test_merge_w_relationship(self):
-        ajoin = polymorphic_union({'a': a_table, 'b': b_table, 'c':c_table}, 
+        A, C, B, c_table, b_table, a_table, Dest, dest_table = (self.classes.A,
+                                self.classes.C,
+                                self.classes.B,
+                                self.tables.c_table,
+                                self.tables.b_table,
+                                self.tables.a_table,
+                                self.classes.Dest,
+                                self.tables.dest_table)
+
+        ajoin = polymorphic_union({'a': a_table, 'b': b_table, 'c':c_table},
                                 'type','ajoin')
         mapper(
             A,
@@ -579,7 +618,7 @@ class PropertyInheritanceTest(_base.MappedTest):
 
         mapper(Dest, dest_table, properties={
                 'many_a': relationship(A,
-                            back_populates='some_dest', 
+                            back_populates='some_dest',
                             order_by=ajoin.c.id)
                         }
                 )
@@ -602,7 +641,7 @@ class PropertyInheritanceTest(_base.MappedTest):
         eq_(merged_c1.some_dest.name, 'd2')
         eq_(merged_c1.some_dest_id, c1.some_dest_id)
 
-class ManyToManyTest(_base.MappedTest):
+class ManyToManyTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
@@ -622,20 +661,27 @@ class ManyToManyTest(_base.MappedTest):
               primary_key=True, test_needs_autoincrement=True))
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_classes(cls):
-        class Base(_base.ComparableEntity):
+        class Base(cls.Comparable):
             pass
 
         class Sub(Base):
             pass
 
-        class Related(_base.ComparableEntity):
+        class Related(cls.Comparable):
             pass
 
 
-    @testing.resolve_artifact_names
     def test_selective_relationships(self):
+        sub, base_mtom, Related, Base, related, sub_mtom, base, Sub = (self.tables.sub,
+                                self.tables.base_mtom,
+                                self.classes.Related,
+                                self.classes.Base,
+                                self.tables.related,
+                                self.tables.sub_mtom,
+                                self.tables.base,
+                                self.classes.Sub)
+
         mapper(Base, base, properties={'related': relationship(Related,
                secondary=base_mtom, backref='bases',
                order_by=related.c.id)})
@@ -657,7 +703,7 @@ class ManyToManyTest(_base.MappedTest):
         eq_(b1.related, [r1, r2])
 
 
-class ColKeysTest(_base.MappedTest):
+class ColKeysTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
