@@ -9,12 +9,12 @@ be cleaned up and modernized.
 import datetime
 
 import sqlalchemy as sa
-from test.lib import testing
+from sqlalchemy import testing
 from sqlalchemy import Integer, String, ForeignKey
-from test.lib.schema import Table, Column
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, backref, create_session
-from test.lib.testing import eq_
-from test.lib import fixtures
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing import fixtures
 
 
 class EagerTest(fixtures.MappedTest):
@@ -120,8 +120,8 @@ class EagerTest(fixtures.MappedTest):
                                 self.tables.categories)
 
         # I want to display a list of tests owned by owner 1
-        # if someoption is false or he hasn't specified it yet (null)
-        # but not if he set it to true (example someoption is for hiding)
+        # if someoption is false or they haven't specified it yet (null)
+        # but not if they set it to true (example someoption is for hiding)
 
         # desired output for owner 1
         # test_id, cat_name
