@@ -1,14 +1,16 @@
 
-from test.lib.testing import assert_raises, assert_raises_message
+from sqlalchemy.testing import assert_raises, assert_raises_message
 from sqlalchemy import Integer, String, ForeignKey, Sequence, \
     exc as sa_exc
-from test.lib.schema import Table, Column
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, create_session, \
-    sessionmaker, class_mapper, backref, Session, util as orm_util
-from sqlalchemy.orm import attributes, exc as orm_exc
-from test.lib import testing
-from test.lib.testing import eq_
-from test.lib import fixtures
+    sessionmaker, class_mapper, backref, Session, util as orm_util,\
+    configure_mappers
+from sqlalchemy.orm.attributes import instance_state
+from sqlalchemy.orm import attributes, exc as orm_exc, object_mapper
+from sqlalchemy import testing
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
 
 class CascadeArgTest(fixtures.MappedTest):
@@ -35,6 +37,22 @@ class CascadeArgTest(fixtures.MappedTest):
         class Address(cls.Basic):
             pass
 
+    def test_delete_with_passive_deletes_all(self):
+        User, Address = self.classes.User, self.classes.Address
+        users, addresses = self.tables.users, self.tables.addresses
+
+        mapper(User, users, properties={
+            'addresses': relationship(Address,
+                    passive_deletes="all", cascade="all, delete-orphan")
+            })
+        mapper(Address, addresses)
+        assert_raises_message(
+            sa_exc.ArgumentError,
+            "On User.addresses, can't set passive_deletes='all' "
+            "in conjunction with 'delete' or 'delete-orphan' cascade",
+            configure_mappers
+        )
+
     def test_delete_orphan_without_delete(self):
         User, Address = self.classes.User, self.classes.Address
         users, addresses = self.tables.users, self.tables.addresses
@@ -67,6 +85,33 @@ class CascadeArgTest(fixtures.MappedTest):
             orm_util.CascadeOptions("all, delete-orphan"),
             frozenset)
 
+    def test_cascade_assignable(self):
+        User, Address = self.classes.User, self.classes.Address
+        users, addresses = self.tables.users, self.tables.addresses
+
+        rel = relationship(Address)
+        eq_(rel.cascade, set(['save-update', 'merge']))
+        rel.cascade = "save-update, merge, expunge"
+        eq_(rel.cascade, set(['save-update', 'merge', 'expunge']))
+
+        mapper(User, users, properties={
+                'addresses': rel
+            })
+        am = mapper(Address, addresses)
+        configure_mappers()
+
+        eq_(rel.cascade, set(['save-update', 'merge', 'expunge']))
+
+        assert ("addresses", User) not in am._delete_orphans
+        rel.cascade = "all, delete, delete-orphan"
+        assert ("addresses", User) in am._delete_orphans
+
+        eq_(rel.cascade,
+            set(['delete', 'delete-orphan', 'expunge', 'merge',
+                    'refresh-expire', 'save-update'])
+            )
+
+
 class O2MCascadeDeleteOrphanTest(fixtures.MappedTest):
     run_inserts = None
 
@@ -396,20 +441,20 @@ class O2MCascadeTest(fixtures.MappedTest):
 
         })
 
-    def test_none_skipped_assignment(self):
-        # [ticket:2229] proposes warning/raising on None
-        # for 0.8
+    def test_none_o2m_collection_assignment(self):
         User, Address = self.classes.User, self.classes.Address
         s = Session()
         u1 = User(name='u', addresses=[None])
         s.add(u1)
         eq_(u1.addresses, [None])
-        s.commit()
-        eq_(u1.addresses, [])
+        assert_raises_message(
+            orm_exc.FlushError,
+            "Can't flush None value found in collection User.addresses",
+            s.commit
+        )
+        eq_(u1.addresses, [None])
 
-    def test_none_skipped_append(self):
-        # [ticket:2229] proposes warning/raising on None
-        # for 0.8
+    def test_none_o2m_collection_append(self):
         User, Address = self.classes.User, self.classes.Address
         s = Session()
 
@@ -417,8 +462,13 @@ class O2MCascadeTest(fixtures.MappedTest):
         s.add(u1)
         u1.addresses.append(None)
         eq_(u1.addresses, [None])
-        s.commit()
-        eq_(u1.addresses, [])
+        assert_raises_message(
+            orm_exc.FlushError,
+            "Can't flush None value found in collection User.addresses",
+            s.commit
+        )
+        eq_(u1.addresses, [None])
+
 
 class O2MCascadeDeleteNoOrphanTest(fixtures.MappedTest):
     run_inserts = None
@@ -1725,6 +1775,24 @@ class M2MCascadeTest(fixtures.MappedTest):
         assert b.count().scalar() == 0
         assert a.count().scalar() == 0
 
+    def test_single_parent_error(self):
+        a, A, B, b, atob = (self.tables.a,
+                                self.classes.A,
+                                self.classes.B,
+                                self.tables.b,
+                                self.tables.atob)
+
+        mapper(A, a, properties={
+            'bs':relationship(B, secondary=atob,
+                        cascade="all, delete-orphan")
+        })
+        mapper(B, b)
+        assert_raises_message(
+            sa_exc.ArgumentError,
+            "On A.bs, delete-orphan cascade is not supported",
+            configure_mappers
+        )
+
     def test_single_parent_raise(self):
         a, A, B, b, atob = (self.tables.a,
                                 self.classes.A,
@@ -1778,6 +1846,57 @@ class M2MCascadeTest(fixtures.MappedTest):
         assert b1 not in a1.bs
         assert b1 in a2.bs
 
+    def test_none_m2m_collection_assignment(self):
+        a, A, B, b, atob = (self.tables.a,
+                                self.classes.A,
+                                self.classes.B,
+                                self.tables.b,
+                                self.tables.atob)
+
+
+        mapper(A, a, properties={
+            'bs': relationship(B,
+                secondary=atob, backref="as")
+        })
+        mapper(B, b)
+
+        s = Session()
+        a1 = A(bs=[None])
+        s.add(a1)
+        eq_(a1.bs, [None])
+        assert_raises_message(
+            orm_exc.FlushError,
+            "Can't flush None value found in collection A.bs",
+            s.commit
+        )
+        eq_(a1.bs, [None])
+
+    def test_none_m2m_collection_append(self):
+        a, A, B, b, atob = (self.tables.a,
+                                self.classes.A,
+                                self.classes.B,
+                                self.tables.b,
+                                self.tables.atob)
+
+
+        mapper(A, a, properties={
+            'bs': relationship(B,
+                secondary=atob, backref="as")
+        })
+        mapper(B, b)
+
+        s = Session()
+        a1 = A()
+        a1.bs.append(None)
+        s.add(a1)
+        eq_(a1.bs, [None])
+        assert_raises_message(
+            orm_exc.FlushError,
+            "Can't flush None value found in collection A.bs",
+            s.commit
+        )
+        eq_(a1.bs, [None])
+
 class O2MSelfReferentialDetelOrphanTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
@@ -2186,10 +2305,7 @@ class DoubleParentO2MOrphanTest(fixtures.MappedTest):
             Column('account_id', Integer,
                    ForeignKey('accounts.account_id')))
 
-    def test_double_parent_expunge_o2m(self):
-        """test the delete-orphan uow event for multiple delete-orphan
-        parent relationships."""
-
+    def _fixture(self, legacy_is_orphan, uselist):
         sales_reps, customers, accounts = (self.tables.sales_reps,
                                 self.tables.customers,
                                 self.tables.accounts)
@@ -2202,15 +2318,17 @@ class DoubleParentO2MOrphanTest(fixtures.MappedTest):
         class SalesRep(fixtures.ComparableEntity):
             pass
 
-        mapper(Customer, customers)
+        mapper(Customer, customers, legacy_is_orphan=legacy_is_orphan)
         mapper(Account, accounts, properties=dict(
             customers=relationship(Customer,
                                cascade="all,delete-orphan",
-                               backref="account")))
+                               backref="account",
+                               uselist=uselist)))
         mapper(SalesRep, sales_reps, properties=dict(
             customers=relationship(Customer,
                                cascade="all,delete-orphan",
-                               backref="sales_rep")))
+                               backref="sales_rep",
+                               uselist=uselist)))
         s = create_session()
 
         a = Account(balance=0)
@@ -2220,9 +2338,21 @@ class DoubleParentO2MOrphanTest(fixtures.MappedTest):
 
         c = Customer(name="Jane")
 
-        a.customers.append(c)
-        sr.customers.append(c)
+        if uselist:
+            a.customers.append(c)
+            sr.customers.append(c)
+        else:
+            a.customers = c
+            sr.customers = c
+
         assert c in s
+        return s, c, a, sr
+
+    def test_double_parent_expunge_o2m_legacy(self):
+        """test the delete-orphan uow event for multiple delete-orphan
+        parent relationships."""
+
+        s, c, a, sr = self._fixture(True, True)
 
         a.customers.remove(c)
         assert c in s, "Should not expunge customer yet, still has one parent"
@@ -2231,48 +2361,44 @@ class DoubleParentO2MOrphanTest(fixtures.MappedTest):
         assert c not in s, \
             'Should expunge customer when both parents are gone'
 
-    def test_double_parent_expunge_o2o(self):
+    def test_double_parent_expunge_o2m_current(self):
         """test the delete-orphan uow event for multiple delete-orphan
         parent relationships."""
 
-        sales_reps, customers, accounts = (self.tables.sales_reps,
-                                self.tables.customers,
-                                self.tables.accounts)
+        s, c, a, sr = self._fixture(False, True)
+
+        a.customers.remove(c)
+        assert c not in s, "Should expunge customer when either parent is gone"
+
+        sr.customers.remove(c)
+        assert c not in s, \
+            'Should expunge customer when both parents are gone'
+
+    def test_double_parent_expunge_o2o_legacy(self):
+        """test the delete-orphan uow event for multiple delete-orphan
+        parent relationships."""
+
+        s, c, a, sr = self._fixture(True, False)
 
 
-        class Customer(fixtures.ComparableEntity):
-            pass
-        class Account(fixtures.ComparableEntity):
-            pass
-        class SalesRep(fixtures.ComparableEntity):
-            pass
-
-        mapper(Customer, customers)
-        mapper(Account, accounts, properties=dict(
-            customer=relationship(Customer,
-                               cascade="all,delete-orphan",
-                               backref="account", uselist=False)))
-        mapper(SalesRep, sales_reps, properties=dict(
-            customer=relationship(Customer,
-                               cascade="all,delete-orphan",
-                               backref="sales_rep", uselist=False)))
-        s = create_session()
-
-        a = Account(balance=0)
-        sr = SalesRep(name="John")
-        s.add_all((a, sr))
-        s.flush()
-
-        c = Customer(name="Jane")
-
-        a.customer = c
-        sr.customer = c
-        assert c in s
-
-        a.customer = None
+        a.customers = None
         assert c in s, "Should not expunge customer yet, still has one parent"
 
-        sr.customer = None
+        sr.customers = None
+        assert c not in s, \
+            'Should expunge customer when both parents are gone'
+
+    def test_double_parent_expunge_o2o_current(self):
+        """test the delete-orphan uow event for multiple delete-orphan
+        parent relationships."""
+
+        s, c, a, sr = self._fixture(False, False)
+
+
+        a.customers = None
+        assert c not in s, "Should expunge customer when either parent is gone"
+
+        sr.customers = None
         assert c not in s, \
             'Should expunge customer when both parents are gone'
 
@@ -2424,6 +2550,160 @@ class CollectionAssignmentOrphanTest(fixtures.MappedTest):
         eq_(sess.query(A).get(a1.id),
             A(name='a1', bs=[B(name='b1'), B(name='b2'), B(name='b3')]))
 
+class OrphanCriterionTest(fixtures.MappedTest):
+    @classmethod
+    def define_tables(self, metadata):
+        Table("core", metadata,
+            Column("id", Integer,
+                    primary_key=True, test_needs_autoincrement=True),
+            Column("related_one_id", Integer, ForeignKey("related_one.id")),
+            Column("related_two_id", Integer, ForeignKey("related_two.id"))
+        )
+
+        Table("related_one", metadata,
+            Column("id", Integer,
+                primary_key=True, test_needs_autoincrement=True),
+            )
+
+        Table("related_two", metadata,
+            Column("id", Integer,
+                primary_key=True, test_needs_autoincrement=True),
+            )
+
+    def _fixture(self, legacy_is_orphan, persistent,
+                        r1_present, r2_present, detach_event=True):
+        class Core(object):
+            pass
+
+        class RelatedOne(object):
+            def __init__(self, cores):
+                self.cores = cores
+
+        class RelatedTwo(object):
+            def __init__(self, cores):
+                self.cores = cores
+
+        mapper(Core, self.tables.core, legacy_is_orphan=legacy_is_orphan)
+        mapper(RelatedOne, self.tables.related_one, properties={
+                'cores': relationship(Core, cascade="all, delete-orphan",
+                    backref="r1")
+            })
+        mapper(RelatedTwo, self.tables.related_two, properties={
+                'cores': relationship(Core, cascade="all, delete-orphan",
+                    backref="r2")
+            })
+        c1 = Core()
+        if detach_event:
+            r1 = RelatedOne(cores=[c1])
+            r2 = RelatedTwo(cores=[c1])
+        else:
+            if r1_present:
+                r1 = RelatedOne(cores=[c1])
+            if r2_present:
+                r2 = RelatedTwo(cores=[c1])
+
+        if persistent:
+            s = Session()
+            s.add(c1)
+            s.flush()
+
+        if detach_event:
+            if not r1_present:
+                c1.r1 = None
+            if not r2_present:
+                c1.r2 = None
+        return c1
+
+    def _assert_not_orphan(self, c1):
+        mapper = object_mapper(c1)
+        state = instance_state(c1)
+        assert not mapper._is_orphan(state)
+
+    def _assert_is_orphan(self, c1):
+        mapper = object_mapper(c1)
+        state = instance_state(c1)
+        assert mapper._is_orphan(state)
+
+    def test_leg_pers_r1_r2(self):
+        c1 = self._fixture(True, True, True, True)
+
+        self._assert_not_orphan(c1)
+
+    def test_current_pers_r1_r2(self):
+        c1 = self._fixture(False, True, True, True)
+
+        self._assert_not_orphan(c1)
+
+    def test_leg_pers_r1_notr2(self):
+        c1 = self._fixture(True, True, True, False)
+
+        self._assert_not_orphan(c1)
+
+    def test_current_pers_r1_notr2(self):
+        c1 = self._fixture(False, True, True, False)
+
+        self._assert_is_orphan(c1)
+
+    def test_leg_pers_notr1_notr2(self):
+        c1 = self._fixture(True, True, False, False)
+
+        self._assert_is_orphan(c1)
+
+    def test_current_pers_notr1_notr2(self):
+        c1 = self._fixture(False, True, True, False)
+
+        self._assert_is_orphan(c1)
+
+    def test_leg_transient_r1_r2(self):
+        c1 = self._fixture(True, False, True, True)
+
+        self._assert_not_orphan(c1)
+
+    def test_current_transient_r1_r2(self):
+        c1 = self._fixture(False, False, True, True)
+
+        self._assert_not_orphan(c1)
+
+    def test_leg_transient_r1_notr2(self):
+        c1 = self._fixture(True, False, True, False)
+
+        self._assert_not_orphan(c1)
+
+    def test_current_transient_r1_notr2(self):
+        c1 = self._fixture(False, False, True, False)
+
+        self._assert_is_orphan(c1)
+
+    def test_leg_transient_notr1_notr2(self):
+        c1 = self._fixture(True, False, False, False)
+
+        self._assert_is_orphan(c1)
+
+    def test_current_transient_notr1_notr2(self):
+        c1 = self._fixture(False, False, False, False)
+
+        self._assert_is_orphan(c1)
+
+    def test_leg_transient_notr1_notr2_noevent(self):
+        c1 = self._fixture(True, False, False, False, False)
+
+        self._assert_is_orphan(c1)
+
+    def test_current_transient_notr1_notr2_noevent(self):
+        c1 = self._fixture(False, False, False, False, False)
+
+        self._assert_is_orphan(c1)
+
+    def test_leg_persistent_notr1_notr2_noevent(self):
+        c1 = self._fixture(True, True, False, False, False)
+
+        self._assert_not_orphan(c1)
+
+    def test_current_persistent_notr1_notr2_noevent(self):
+        c1 = self._fixture(False, True, False, False, False)
+
+        self._assert_not_orphan(c1)
+
 class O2MConflictTest(fixtures.MappedTest):
     """test that O2M dependency detects a change in parent, does the
     right thing, and updates the collection/attribute.
