@@ -1,26 +1,29 @@
-from test.lib.testing import assert_raises_message
+from sqlalchemy.testing import assert_raises_message, assert_raises
 import sqlalchemy as sa
-from test.lib import testing
+from sqlalchemy import testing
 from sqlalchemy import Integer, String
-from test.lib.schema import Table, Column
+from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, \
     create_session, class_mapper, \
     Mapper, column_property, \
-    Session, sessionmaker
+    Session, sessionmaker, attributes
 from sqlalchemy.orm.instrumentation import ClassManager
-from test.lib.testing import eq_
-from test.lib import fixtures
+from sqlalchemy.orm import instrumentation, events
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.util import gc_collect
 from test.orm import _fixtures
 from sqlalchemy import event
-
+from sqlalchemy.testing.mock import Mock, call
 
 class _RemoveListeners(object):
     def teardown(self):
         # TODO: need to get remove() functionality
         # going
-        Mapper.dispatch._clear()
-        ClassManager.dispatch._clear()
-        Session.dispatch._clear()
+        events.MapperEvents._clear()
+        events.InstanceEvents._clear()
+        events.SessionEvents._clear()
+        events.InstrumentationEvents._clear()
         super(_RemoveListeners, self).teardown()
 
 
@@ -64,13 +67,13 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         event.listen(A, 'init', init_e, propagate=True)
 
         a = A()
-        eq_(canary, [('init_a', a),('init_b', a),
-                        ('init_c', a),('init_d', a),('init_e', a)])
+        eq_(canary, [('init_a', a), ('init_b', a),
+                        ('init_c', a), ('init_d', a), ('init_e', a)])
 
         # test propagate flag
         canary[:] = []
         b = B()
-        eq_(canary, [('init_a', b), ('init_b', b),('init_e', b)])
+        eq_(canary, [('init_a', b), ('init_b', b), ('init_e', b)])
 
 
     def listen_all(self, mapper, **kw):
@@ -100,6 +103,14 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
             event.listen(mapper, meth, evt(meth), **kw)
         return canary
 
+    def test_listen_doesnt_force_compile(self):
+        User, users = self.classes.User, self.tables.users
+        m = mapper(User, users, properties={
+            'addresses': relationship(lambda: ImNotAClass)
+        })
+        event.listen(User, "before_insert", lambda *a, **kw: None)
+        assert not m.configured
+
     def test_basic(self):
         User, users = self.classes.User, self.tables.users
 
@@ -121,18 +132,19 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         sess.flush()
         eq_(canary,
             ['init', 'before_insert',
-             'after_insert', 'expire', 'translate_row', 'populate_instance',
-             'refresh',
+             'after_insert', 'expire', 'translate_row',
+             'populate_instance', 'refresh',
              'append_result', 'translate_row', 'create_instance',
              'populate_instance', 'load', 'append_result',
-             'before_update', 'after_update', 'before_delete', 'after_delete'])
+             'before_update', 'after_update', 'before_delete',
+             'after_delete'])
 
     def test_merge(self):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
 
-        canary =[]
+        canary = []
         def load(obj, ctx):
             canary.append('load')
         event.listen(mapper, 'load', load)
@@ -147,7 +159,7 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         u2 = s.merge(User(name='u2'))
         s.commit()
         s.query(User).first()
-        eq_(canary,['load', 'load', 'load'])
+        eq_(canary, ['load', 'load', 'load'])
 
     def test_inheritance(self):
         users, addresses, User = (self.tables.users,
@@ -189,13 +201,55 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
             'before_update', 'after_update', 'before_delete',
             'after_delete'])
 
+    def test_inheritance_subclass_deferred(self):
+        users, addresses, User = (self.tables.users,
+                                self.tables.addresses,
+                                self.classes.User)
+
+
+        mapper(User, users)
+
+        canary1 = self.listen_all(User, propagate=True)
+        canary2 = self.listen_all(User)
+
+        class AdminUser(User):
+            pass
+        mapper(AdminUser, addresses, inherits=User)
+        canary3 = self.listen_all(AdminUser)
+
+        sess = create_session()
+        am = AdminUser(name='au1', email_address='au1@e1')
+        sess.add(am)
+        sess.flush()
+        am = sess.query(AdminUser).populate_existing().get(am.id)
+        sess.expunge_all()
+        am = sess.query(AdminUser).get(am.id)
+        am.name = 'au1 changed'
+        sess.flush()
+        sess.delete(am)
+        sess.flush()
+        eq_(canary1, ['init', 'before_insert', 'after_insert',
+            'translate_row', 'populate_instance','refresh',
+            'append_result', 'translate_row', 'create_instance'
+            , 'populate_instance', 'load', 'append_result',
+            'before_update', 'after_update', 'before_delete',
+            'after_delete'])
+        eq_(canary2, [])
+        eq_(canary3, ['init', 'before_insert', 'after_insert',
+            'translate_row', 'populate_instance','refresh',
+            'append_result', 'translate_row', 'create_instance'
+            , 'populate_instance', 'load', 'append_result',
+            'before_update', 'after_update', 'before_delete',
+            'after_delete'])
+
     def test_before_after_only_collection(self):
         """before_update is called on parent for collection modifications,
         after_update is called even if no columns were updated.
 
         """
 
-        keywords, items, item_keywords, Keyword, Item = (self.tables.keywords,
+        keywords, items, item_keywords, Keyword, Item = (
+                                self.tables.keywords,
                                 self.tables.items,
                                 self.tables.item_keywords,
                                 self.classes.Keyword,
@@ -216,10 +270,10 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         sess.add(k1)
         sess.flush()
         eq_(canary1,
-            ['init', 
+            ['init',
             'before_insert', 'after_insert'])
         eq_(canary2,
-            ['init', 
+            ['init',
             'before_insert', 'after_insert'])
 
         canary1[:]= []
@@ -267,6 +321,351 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         mapper(Address, addresses)
         eq_(canary, [User, Address])
 
+class DeferredMapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
+    """"test event listeners against unmapped classes.
+
+    This incurs special logic.  Note if we ever do the "remove" case,
+    it has to get all of these, too.
+
+    """
+    run_inserts = None
+
+    def test_deferred_map_event(self):
+        """
+        1. mapper event listen on class
+        2. map class
+        3. event fire should receive event
+
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "before_insert", evt, raw=True)
+
+        m = mapper(User, users)
+        m.dispatch.before_insert(5)
+        eq_(canary, [5])
+
+    def test_deferred_map_event_subclass_propagate(self):
+        """
+        1. mapper event listen on class, w propagate
+        2. map only subclass of class
+        3. event fire should receive event
+
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        class SubUser(User):
+            pass
+
+        class SubSubUser(SubUser):
+            pass
+
+        canary = Mock()
+        def evt(x, y, z):
+            canary.append(x)
+        event.listen(User, "before_insert", canary, propagate=True, raw=True)
+
+        m = mapper(SubUser, users)
+        m.dispatch.before_insert(5, 6, 7)
+        eq_(canary.mock_calls,
+                [call(5, 6, 7)])
+
+        m2 = mapper(SubSubUser, users)
+
+        m2.dispatch.before_insert(8, 9, 10)
+        eq_(canary.mock_calls,
+                [call(5, 6, 7), call(8, 9, 10)])
+
+
+    def test_deferred_map_event_subclass_no_propagate(self):
+        """
+        1. mapper event listen on class, w/o propagate
+        2. map only subclass of class
+        3. event fire should not receive event
+
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        class SubUser(User):
+            pass
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "before_insert", evt, propagate=False)
+
+        m = mapper(SubUser, users)
+        m.dispatch.before_insert(5)
+        eq_(canary, [])
+
+    def test_deferred_map_event_subclass_post_mapping_propagate(self):
+        """
+        1. map only subclass of class
+        2. mapper event listen on class, w propagate
+        3. event fire should receive event
+
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        class SubUser(User):
+            pass
+
+        m = mapper(SubUser, users)
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "before_insert", evt, propagate=True, raw=True)
+
+        m.dispatch.before_insert(5)
+        eq_(canary, [5])
+
+    def test_deferred_map_event_subclass_post_mapping_propagate_two(self):
+        """
+        1. map only subclass of class
+        2. mapper event listen on class, w propagate
+        3. event fire should receive event
+
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        class SubUser(User):
+            pass
+
+        class SubSubUser(SubUser):
+            pass
+
+        m = mapper(SubUser, users)
+
+        canary = Mock()
+        event.listen(User, "before_insert", canary, propagate=True, raw=True)
+
+        m2 = mapper(SubSubUser, users)
+
+        m.dispatch.before_insert(5, 6, 7)
+        eq_(canary.mock_calls, [call(5, 6, 7)])
+
+        m2.dispatch.before_insert(8, 9, 10)
+        eq_(canary.mock_calls, [call(5, 6, 7), call(8, 9, 10)])
+
+    def test_deferred_instance_event_subclass_post_mapping_propagate(self):
+        """
+        1. map only subclass of class
+        2. instance event listen on class, w propagate
+        3. event fire should receive event
+
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        class SubUser(User):
+            pass
+
+        m = mapper(SubUser, users)
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "load", evt, propagate=True, raw=True)
+
+        m.class_manager.dispatch.load(5)
+        eq_(canary, [5])
+
+
+    def test_deferred_instance_event_plain(self):
+        """
+        1. instance event listen on class, w/o propagate
+        2. map class
+        3. event fire should receive event
+
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "load", evt, raw=True)
+
+        m = mapper(User, users)
+        m.class_manager.dispatch.load(5)
+        eq_(canary, [5])
+
+    def test_deferred_instance_event_subclass_propagate_subclass_only(self):
+        """
+        1. instance event listen on class, w propagate
+        2. map two subclasses of class
+        3. event fire on each class should receive one and only one event
+
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        class SubUser(User):
+            pass
+
+        class SubUser2(User):
+            pass
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "load", evt, propagate=True, raw=True)
+
+        m = mapper(SubUser, users)
+        m2 = mapper(SubUser2, users)
+
+        m.class_manager.dispatch.load(5)
+        eq_(canary, [5])
+
+        m2.class_manager.dispatch.load(5)
+        eq_(canary, [5, 5])
+
+    def test_deferred_instance_event_subclass_propagate_baseclass(self):
+        """
+        1. instance event listen on class, w propagate
+        2. map one subclass of class, map base class, leave 2nd subclass unmapped
+        3. event fire on sub should receive one and only one event
+        4. event fire on base should receive one and only one event
+        5. map 2nd subclass
+        6. event fire on 2nd subclass should receive one and only one event
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        class SubUser(User):
+            pass
+
+        class SubUser2(User):
+            pass
+
+        canary = Mock()
+        event.listen(User, "load", canary, propagate=True, raw=False)
+
+        # reversing these fixes....
+        m = mapper(SubUser, users)
+        m2 = mapper(User, users)
+
+        instance = Mock()
+        m.class_manager.dispatch.load(instance)
+
+        eq_(canary.mock_calls, [call(instance.obj())])
+
+        m2.class_manager.dispatch.load(instance)
+        eq_(canary.mock_calls, [call(instance.obj()), call(instance.obj())])
+
+        m3 = mapper(SubUser2, users)
+        m3.class_manager.dispatch.load(instance)
+        eq_(canary.mock_calls, [call(instance.obj()),
+                        call(instance.obj()), call(instance.obj())])
+
+    def test_deferred_instance_event_subclass_no_propagate(self):
+        """
+        1. instance event listen on class, w/o propagate
+        2. map subclass
+        3. event fire on subclass should not receive event
+        """
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        class SubUser(User):
+            pass
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "load", evt, propagate=False)
+
+        m = mapper(SubUser, users)
+        m.class_manager.dispatch.load(5)
+        eq_(canary, [])
+
+    def test_deferred_instrument_event(self):
+        users, User = (self.tables.users,
+                                self.classes.User)
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "attribute_instrument", evt)
+
+        instrumentation._instrumentation_factory.dispatch.attribute_instrument(User)
+        eq_(canary, [User])
+
+    def test_isolation_instrument_event(self):
+        users, User = (self.tables.users,
+                                self.classes.User)
+        class Bar(object):
+            pass
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(Bar, "attribute_instrument", evt)
+
+        instrumentation._instrumentation_factory.dispatch.attribute_instrument(User)
+        eq_(canary, [])
+
+    @testing.requires.predictable_gc
+    def test_instrument_event_auto_remove(self):
+        class Bar(object):
+            pass
+
+        listeners = instrumentation._instrumentation_factory.dispatch.\
+                            attribute_instrument.listeners
+        assert not listeners
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(Bar, "attribute_instrument", evt)
+
+        eq_(len(listeners), 1)
+
+        del Bar
+        gc_collect()
+
+        assert not listeners
+
+
+    def test_deferred_instrument_event_subclass_propagate(self):
+        users, User = (self.tables.users,
+                                self.classes.User)
+        class SubUser(User):
+            pass
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "attribute_instrument", evt, propagate=True)
+
+        instrumentation._instrumentation_factory.dispatch.\
+                            attribute_instrument(SubUser)
+        eq_(canary, [SubUser])
+
+    def test_deferred_instrument_event_subclass_no_propagate(self):
+        users, User = (self.tables.users,
+                                self.classes.User)
+        class SubUser(User):
+            pass
+
+        canary = []
+        def evt(x):
+            canary.append(x)
+        event.listen(User, "attribute_instrument", evt, propagate=False)
+
+        mapper(SubUser, users)
+        instrumentation._instrumentation_factory.dispatch.attribute_instrument(5)
+        eq_(canary, [])
+
 
 class LoadTest(_fixtures.FixtureTest):
     run_inserts = None
@@ -458,7 +857,8 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         assert my_listener in s.dispatch.before_flush
 
     def test_sessionmaker_listen(self):
-        """test that listen can be applied to individual scoped_session() classes."""
+        """test that listen can be applied to individual
+        scoped_session() classes."""
 
         def my_listener_one(*arg, **kw):
             pass
@@ -489,8 +889,8 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
         assert_raises_message(
             sa.exc.ArgumentError,
-            "Session event listen on a ScopedSession "
-            "requires that its creation callable is a Session subclass.",
+            "Session event listen on a scoped_session requires that its "
+            "creation callable is associated with the Session class.",
             event.listen, scope, "before_flush", my_listener_one
         )
 
@@ -508,8 +908,8 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
         assert_raises_message(
             sa.exc.ArgumentError,
-            "Session event listen on a ScopedSession "
-            "requires that its creation callable is a Session subclass.",
+            "Session event listen on a scoped_session requires that its "
+            "creation callable is associated with the Session class.",
             event.listen, scope, "before_flush", my_listener_one
         )
 
@@ -534,13 +934,17 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         sess = Session(**kw)
 
         for evt in [
+            'after_transaction_create',
+            'after_transaction_end',
             'before_commit',
             'after_commit',
             'after_rollback',
+            'after_soft_rollback',
             'before_flush',
             'after_flush',
             'after_flush_postexec',
             'after_begin',
+            'before_attach',
             'after_attach',
             'after_bulk_update',
             'after_bulk_delete'
@@ -555,18 +959,75 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
         mapper(User, users)
 
-        sess, canary = self._listener_fixture(autoflush=False, autocommit=True, expire_on_commit=False)
+        sess, canary = self._listener_fixture(autoflush=False,
+                            autocommit=True, expire_on_commit=False)
 
         u = User(name='u1')
         sess.add(u)
         sess.flush()
         eq_(
-            canary, 
-            [ 'after_attach', 'before_flush', 'after_begin',
-            'after_flush', 'after_flush_postexec', 
-            'before_commit', 'after_commit',]
+            canary,
+            [ 'before_attach', 'after_attach', 'before_flush',
+            'after_transaction_create', 'after_begin',
+            'after_flush', 'after_flush_postexec',
+            'before_commit', 'after_commit','after_transaction_end']
         )
 
+    def test_rollback_hook(self):
+        User, users = self.classes.User, self.tables.users
+        sess, canary = self._listener_fixture()
+        mapper(User, users)
+
+        u = User(name='u1', id=1)
+        sess.add(u)
+        sess.commit()
+
+        u2 = User(name='u1', id=1)
+        sess.add(u2)
+        assert_raises(
+            sa.orm.exc.FlushError,
+            sess.commit
+        )
+        sess.rollback()
+        eq_(canary,
+
+        ['before_attach', 'after_attach', 'before_commit', 'before_flush',
+        'after_transaction_create', 'after_begin', 'after_flush',
+        'after_flush_postexec', 'after_transaction_end', 'after_commit',
+        'after_transaction_end', 'after_transaction_create',
+        'before_attach', 'after_attach', 'before_commit',
+        'before_flush', 'after_transaction_create', 'after_begin', 'after_rollback',
+        'after_transaction_end',
+        'after_soft_rollback', 'after_transaction_end','after_transaction_create',
+        'after_soft_rollback'])
+
+    def test_can_use_session_in_outer_rollback_hook(self):
+        User, users = self.classes.User, self.tables.users
+        mapper(User, users)
+
+        sess = Session()
+
+        assertions = []
+        @event.listens_for(sess, "after_soft_rollback")
+        def do_something(session, previous_transaction):
+            if session.is_active:
+                assertions.append('name' not in u.__dict__)
+                assertions.append(u.name == 'u1')
+
+        u = User(name='u1', id=1)
+        sess.add(u)
+        sess.commit()
+
+        u2 = User(name='u1', id=1)
+        sess.add(u2)
+        assert_raises(
+            sa.orm.exc.FlushError,
+            sess.commit
+        )
+        sess.rollback()
+        eq_(assertions, [True, True])
+
+
     def test_flush_noautocommit_hook(self):
         User, users = self.classes.User, self.tables.users
 
@@ -577,8 +1038,10 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         u = User(name='u1')
         sess.add(u)
         sess.flush()
-        eq_(canary, ['after_attach', 'before_flush', 'after_begin',
-                       'after_flush', 'after_flush_postexec'])
+        eq_(canary, ['before_attach', 'after_attach', 'before_flush',
+            'after_transaction_create', 'after_begin',
+                       'after_flush', 'after_flush_postexec',
+                       'after_transaction_end'])
 
     def test_flush_in_commit_hook(self):
         User, users = self.classes.User, self.tables.users
@@ -593,13 +1056,56 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
         u.name = 'ed'
         sess.commit()
-        eq_(canary, ['before_commit', 'before_flush', 'after_flush',
-                       'after_flush_postexec', 'after_commit'])
+        eq_(canary, ['before_commit', 'before_flush', 'after_transaction_create', 'after_flush',
+                       'after_flush_postexec',
+                       'after_transaction_end',
+                       'after_commit',
+                       'after_transaction_end', 'after_transaction_create',])
+
+    def test_state_before_attach(self):
+        User, users = self.classes.User, self.tables.users
+        sess = Session()
+
+        @event.listens_for(sess, "before_attach")
+        def listener(session, inst):
+            state = attributes.instance_state(inst)
+            if state.key:
+                assert state.key not in session.identity_map
+            else:
+                assert inst not in session.new
+
+        mapper(User, users)
+        u= User(name='u1')
+        sess.add(u)
+        sess.flush()
+        sess.expunge(u)
+        sess.add(u)
+
+    def test_state_after_attach(self):
+        User, users = self.classes.User, self.tables.users
+        sess = Session()
+
+        @event.listens_for(sess, "after_attach")
+        def listener(session, inst):
+            state = attributes.instance_state(inst)
+            if state.key:
+                assert session.identity_map[state.key] is inst
+            else:
+                assert inst in session.new
+
+        mapper(User, users)
+        u= User(name='u1')
+        sess.add(u)
+        sess.flush()
+        sess.expunge(u)
+        sess.add(u)
 
     def test_standalone_on_commit_hook(self):
         sess, canary = self._listener_fixture()
         sess.commit()
-        eq_(canary, ['before_commit', 'after_commit'])
+        eq_(canary, ['before_commit', 'after_commit',
+                'after_transaction_end',
+                'after_transaction_create'])
 
     def test_on_bulk_update_hook(self):
         User, users = self.classes.User, self.tables.users
@@ -659,7 +1165,7 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         u = User(name='u1')
         sess.add(u)
         sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
+        eq_(sess.query(User).order_by(User.name).all(),
             [
                 User(name='another u1'),
                 User(name='u1')
@@ -667,7 +1173,7 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         )
 
         sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
+        eq_(sess.query(User).order_by(User.name).all(),
             [
                 User(name='another u1'),
                 User(name='u1')
@@ -676,7 +1182,7 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
         u.name='u2'
         sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
+        eq_(sess.query(User).order_by(User.name).all(),
             [
                 User(name='another u1'),
                 User(name='another u2'),
@@ -686,7 +1192,7 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
         sess.delete(u)
         sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
+        eq_(sess.query(User).order_by(User.name).all(),
             [
                 User(name='another u1'),
             ]
@@ -707,14 +1213,14 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         u = User(name='u1')
         sess.add(u)
         sess.flush()
-        eq_(sess.query(User).order_by(User.name).all(), 
+        eq_(sess.query(User).order_by(User.name).all(),
             [User(name='u1')]
         )
 
         sess.add(User(name='u2'))
         sess.flush()
         sess.expunge_all()
-        eq_(sess.query(User).order_by(User.name).all(), 
+        eq_(sess.query(User).order_by(User.name).all(),
             [
                 User(name='u1 modified'),
                 User(name='u2')
@@ -724,7 +1230,7 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
 
 class MapperExtensionTest(_fixtures.FixtureTest):
-    """Superseded by MapperEventsTest - test backwards 
+    """Superseded by MapperEventsTest - test backwards
     compatibility of MapperExtension."""
 
     run_inserts = None
@@ -876,10 +1382,10 @@ class MapperExtensionTest(_fixtures.FixtureTest):
         sess.add(k1)
         sess.flush()
         eq_(methods1,
-            ['instrument_class', 'init_instance', 
+            ['instrument_class', 'init_instance',
             'before_insert', 'after_insert'])
         eq_(methods2,
-            ['instrument_class', 'init_instance', 
+            ['instrument_class', 'init_instance',
             'before_insert', 'after_insert'])
 
         del methods1[:]
@@ -959,7 +1465,7 @@ class MapperExtensionTest(_fixtures.FixtureTest):
 class AttributeExtensionTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
-        Table('t1', 
+        Table('t1',
             metadata,
             Column('id', Integer, primary_key=True),
             Column('type', String(40)),
@@ -1012,7 +1518,7 @@ class AttributeExtensionTest(fixtures.MappedTest):
         eq_(b1.data, 'ex1b2')
         eq_(c1.data, 'ex2c2')
 
-        eq_(ext_msg, ["Ex1 'a1'", "Ex1 'b1'", "Ex2 'c1'", 
+        eq_(ext_msg, ["Ex1 'a1'", "Ex1 'b1'", "Ex2 'c1'",
                     "Ex1 'a2'", "Ex1 'b2'", "Ex2 'c2'"])
 
 
