@@ -1,15 +1,15 @@
-from test.lib.testing import eq_, is_, is_not_
-from test.lib import testing
-from test.lib.schema import Table, Column
-from sqlalchemy import Integer, String, ForeignKey, bindparam
+from sqlalchemy.testing import eq_, is_, is_not_
+from sqlalchemy import testing
+from sqlalchemy.testing.schema import Table, Column
+from sqlalchemy import Integer, String, ForeignKey, bindparam, inspect
 from sqlalchemy.orm import backref, subqueryload, subqueryload_all, \
     mapper, relationship, clear_mappers, create_session, lazyload, \
     aliased, joinedload, deferred, undefer, eagerload_all,\
     Session
-from test.lib.testing import eq_, assert_raises, \
+from sqlalchemy.testing import eq_, assert_raises, \
     assert_raises_message
-from test.lib.assertsql import CompiledSQL
-from test.lib import fixtures
+from sqlalchemy.testing.assertsql import CompiledSQL
+from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
 import sqlalchemy as sa
 
@@ -358,8 +358,6 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
         ("subqueryload", "subqueryload", "subqueryload", 4),
         ("subqueryload", "subqueryload", "joinedload", 3),
     ]
-#    _pathing_runs = [("subqueryload", "subqueryload", "joinedload", 3)]
-#    _pathing_runs = [("subqueryload", "subqueryload", "subqueryload", 4)]
 
     def test_options_pathing(self):
         self._do_options_test(self._pathing_runs)
@@ -671,6 +669,26 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
         l = q.order_by(sa.desc(User.id)).limit(2).offset(2).all()
         eq_(list(reversed(self.static.user_all_result[0:2])), l)
 
+    def test_mapper_order_by(self):
+        users, User, Address, addresses = (self.tables.users,
+                                self.classes.User,
+                                self.classes.Address,
+                                self.tables.addresses)
+
+        mapper(Address, addresses)
+        mapper(User, users, properties={
+            'addresses':relationship(Address,
+                            lazy='subquery',
+                            order_by=addresses.c.id),
+        },order_by=users.c.id.desc())
+
+        sess = create_session()
+        q = sess.query(User)
+
+        l = q.limit(2).all()
+        eq_(l, list(reversed(self.static.user_address_result[2:4])))
+
+
     def test_one_to_many_scalar(self):
         Address, addresses, users, User = (self.classes.Address,
                                 self.tables.addresses,
@@ -791,7 +809,7 @@ class LoadOnExistingTest(_fixtures.FixtureTest):
         User, Order, Item = self.classes.User, \
             self.classes.Order, self.classes.Item
         mapper(User, self.tables.users, properties={
-            'orders':relationship(Order), 
+            'orders':relationship(Order),
         })
         mapper(Order, self.tables.orders, properties={
             'items':relationship(Item, secondary=self.tables.order_items),
@@ -841,6 +859,18 @@ class LoadOnExistingTest(_fixtures.FixtureTest):
         self.assert_sql_count(testing.db, go, 1)
         assert 'addresses' not in u1.__dict__
 
+    def test_populate_existing_propagate(self):
+        User, Address, sess = self._eager_config_fixture()
+        u1 = sess.query(User).get(8)
+        u1.addresses[2].email_address = "foofoo"
+        del u1.addresses[1]
+        u1 = sess.query(User).populate_existing().filter_by(id=8).one()
+        # collection is reverted
+        eq_(len(u1.addresses), 3)
+
+        # attributes on related items reverted
+        eq_(u1.addresses[2].email_address, "ed@lala.com")
+
     def test_loads_second_level_collection_to_scalar(self):
         User, Address, Dingaling, sess = self._collection_to_scalar_fixture()
 
@@ -946,6 +976,166 @@ class OrderBySecondaryTest(fixtures.MappedTest):
             ])
         self.assert_sql_count(testing.db, go, 2)
 
+
+from .inheritance._poly_fixtures import _Polymorphic, Person, Engineer, Paperwork
+
+class BaseRelationFromJoinedSubclassTest(_Polymorphic):
+    @classmethod
+    def define_tables(cls, metadata):
+        people = Table('people', metadata,
+            Column('person_id', Integer,
+                primary_key=True,
+                test_needs_autoincrement=True),
+            Column('name', String(50)),
+            Column('type', String(30)))
+
+        # to test fully, PK of engineers table must be
+        # named differently from that of people
+        engineers = Table('engineers', metadata,
+            Column('engineer_id', Integer,
+                ForeignKey('people.person_id'),
+                primary_key=True),
+            Column('primary_language', String(50)))
+
+        paperwork = Table('paperwork', metadata,
+            Column('paperwork_id', Integer,
+                primary_key=True,
+                test_needs_autoincrement=True),
+            Column('description', String(50)),
+            Column('person_id', Integer,
+                ForeignKey('people.person_id')))
+
+    @classmethod
+    def setup_mappers(cls):
+        people = cls.tables.people
+        engineers = cls.tables.engineers
+        paperwork = cls.tables.paperwork
+
+        mapper(Person, people,
+            polymorphic_on=people.c.type,
+            polymorphic_identity='person',
+            properties={
+                'paperwork': relationship(
+                    Paperwork, order_by=paperwork.c.paperwork_id)})
+
+        mapper(Engineer, engineers,
+            inherits=Person,
+            polymorphic_identity='engineer')
+
+        mapper(Paperwork, paperwork)
+
+    @classmethod
+    def insert_data(cls):
+
+        e1 = Engineer(primary_language="java")
+        e2 = Engineer(primary_language="c++")
+        e1.paperwork = [Paperwork(description="tps report #1"),
+                        Paperwork(description="tps report #2")]
+        e2.paperwork = [Paperwork(description="tps report #3")]
+        sess = create_session()
+        sess.add_all([e1, e2])
+        sess.flush()
+
+    def test_correct_subquery_nofrom(self):
+        sess = create_session()
+        # use Person.paperwork here just to give the least
+        # amount of context
+        q = sess.query(Engineer).\
+                filter(Engineer.primary_language == 'java').\
+                options(subqueryload(Person.paperwork))
+        def go():
+            eq_(q.all()[0].paperwork,
+                    [Paperwork(description="tps report #1"),
+                    Paperwork(description="tps report #2")],
+
+                )
+        self.assert_sql_execution(
+                testing.db,
+                go,
+                CompiledSQL(
+                    "SELECT people.person_id AS people_person_id, "
+                    "people.name AS people_name, people.type AS people_type, "
+                    "engineers.engineer_id AS engineers_engineer_id, "
+                    "engineers.primary_language AS engineers_primary_language "
+                    "FROM people JOIN engineers ON "
+                    "people.person_id = engineers.engineer_id "
+                    "WHERE engineers.primary_language = :primary_language_1",
+                    {"primary_language_1": "java"}
+                ),
+                # ensure we get "people JOIN engineer" here, even though
+                # primary key "people.person_id" is against "Person"
+                # *and* the path comes out as "Person.paperwork", still
+                # want to select from "Engineer" entity
+                CompiledSQL(
+                    "SELECT paperwork.paperwork_id AS paperwork_paperwork_id, "
+                    "paperwork.description AS paperwork_description, "
+                    "paperwork.person_id AS paperwork_person_id, "
+                    "anon_1.people_person_id AS anon_1_people_person_id "
+                    "FROM (SELECT people.person_id AS people_person_id "
+                        "FROM people JOIN engineers "
+                        "ON people.person_id = engineers.engineer_id "
+                        "WHERE engineers.primary_language = "
+                            ":primary_language_1) AS anon_1 "
+                    "JOIN paperwork "
+                        "ON anon_1.people_person_id = paperwork.person_id "
+                    "ORDER BY anon_1.people_person_id, paperwork.paperwork_id",
+                    {"primary_language_1": "java"}
+                )
+        )
+
+    def test_correct_subquery_existingfrom(self):
+        sess = create_session()
+        # use Person.paperwork here just to give the least
+        # amount of context
+        q = sess.query(Engineer).\
+                filter(Engineer.primary_language == 'java').\
+                join(Engineer.paperwork).\
+                filter(Paperwork.description == "tps report #2").\
+                options(subqueryload(Person.paperwork))
+        def go():
+            eq_(q.one().paperwork,
+                    [Paperwork(description="tps report #1"),
+                    Paperwork(description="tps report #2")],
+
+                )
+        self.assert_sql_execution(
+            testing.db,
+            go,
+            CompiledSQL(
+                "SELECT people.person_id AS people_person_id, "
+                "people.name AS people_name, people.type AS people_type, "
+                "engineers.engineer_id AS engineers_engineer_id, "
+                "engineers.primary_language AS engineers_primary_language "
+                "FROM people JOIN engineers "
+                    "ON people.person_id = engineers.engineer_id "
+                    "JOIN paperwork ON people.person_id = paperwork.person_id "
+                "WHERE engineers.primary_language = :primary_language_1 "
+                "AND paperwork.description = :description_1",
+                {"primary_language_1": "java",
+                    "description_1": "tps report #2"}
+            ),
+            CompiledSQL(
+                "SELECT paperwork.paperwork_id AS paperwork_paperwork_id, "
+                "paperwork.description AS paperwork_description, "
+                "paperwork.person_id AS paperwork_person_id, "
+                "anon_1.people_person_id AS anon_1_people_person_id "
+                "FROM (SELECT people.person_id AS people_person_id "
+                "FROM people JOIN engineers ON people.person_id = "
+                "engineers.engineer_id JOIN paperwork "
+                "ON people.person_id = paperwork.person_id "
+                "WHERE engineers.primary_language = :primary_language_1 AND "
+                "paperwork.description = :description_1) AS anon_1 "
+                "JOIN paperwork ON anon_1.people_person_id = "
+                "paperwork.person_id "
+                "ORDER BY anon_1.people_person_id, paperwork.paperwork_id",
+                {"primary_language_1": "java",
+                    "description_1": "tps report #2"}
+            )
+        )
+
+
+
+
 class SelfReferentialTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
@@ -1164,4 +1354,413 @@ class SelfReferentialTest(fixtures.MappedTest):
             ], d)
         self.assert_sql_count(testing.db, go, 4)
 
+class InheritanceToRelatedTest(fixtures.MappedTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('foo', metadata,
+            Column("id", Integer, primary_key=True),
+            Column("type", String(50)),
+            Column("related_id", Integer, ForeignKey("related.id"))
+        )
+        Table("bar", metadata,
+            Column("id", Integer, ForeignKey('foo.id'), primary_key=True),
+        )
+        Table("baz", metadata,
+            Column("id", Integer, ForeignKey('foo.id'), primary_key=True),
+        )
+        Table("related", metadata,
+            Column("id", Integer, primary_key=True),
+        )
 
+    @classmethod
+    def setup_classes(cls):
+        class Foo(cls.Comparable):
+            pass
+        class Bar(Foo):
+            pass
+        class Baz(Foo):
+            pass
+        class Related(cls.Comparable):
+            pass
+
+    @classmethod
+    def fixtures(cls):
+        return dict(
+            foo=[
+                ('id', 'type', 'related_id'),
+                (1, 'bar', 1),
+                (2, 'bar', 2),
+                (3, 'baz', 1),
+                (4, 'baz', 2),
+            ],
+            bar=[
+                ('id', ),
+                (1,),
+                (2,)
+            ],
+            baz=[
+                ('id', ),
+                (3,),
+                (4,)
+            ],
+            related=[
+                ('id', ),
+                (1,),
+                (2,)
+            ]
+        )
+    @classmethod
+    def setup_mappers(cls):
+        mapper(cls.classes.Foo, cls.tables.foo, properties={
+            'related': relationship(cls.classes.Related)
+        }, polymorphic_on=cls.tables.foo.c.type)
+        mapper(cls.classes.Bar, cls.tables.bar, polymorphic_identity='bar',
+                    inherits=cls.classes.Foo)
+        mapper(cls.classes.Baz, cls.tables.baz, polymorphic_identity='baz',
+                    inherits=cls.classes.Foo)
+        mapper(cls.classes.Related, cls.tables.related)
+
+    def test_caches_query_per_base_subq(self):
+        Foo, Bar, Baz, Related = self.classes.Foo, self.classes.Bar, \
+                        self.classes.Baz, self.classes.Related
+        s = Session(testing.db)
+        def go():
+            eq_(
+                s.query(Foo).with_polymorphic([Bar, Baz]).\
+                            order_by(Foo.id).\
+                            options(subqueryload(Foo.related)).all(),
+                [
+                    Bar(id=1, related=Related(id=1)),
+                    Bar(id=2, related=Related(id=2)),
+                    Baz(id=3, related=Related(id=1)),
+                    Baz(id=4, related=Related(id=2))
+                ]
+            )
+        self.assert_sql_count(testing.db, go, 2)
+
+    def test_caches_query_per_base_joined(self):
+        # technically this should be in test_eager_relations
+        Foo, Bar, Baz, Related = self.classes.Foo, self.classes.Bar, \
+                        self.classes.Baz, self.classes.Related
+        s = Session(testing.db)
+        def go():
+            eq_(
+                s.query(Foo).with_polymorphic([Bar, Baz]).\
+                            order_by(Foo.id).\
+                            options(joinedload(Foo.related)).all(),
+                [
+                    Bar(id=1, related=Related(id=1)),
+                    Bar(id=2, related=Related(id=2)),
+                    Baz(id=3, related=Related(id=1)),
+                    Baz(id=4, related=Related(id=2))
+                ]
+            )
+        self.assert_sql_count(testing.db, go, 1)
+
+class CyclicalInheritingEagerTestOne(fixtures.MappedTest):
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('t1', metadata,
+            Column('c1', Integer, primary_key=True, test_needs_autoincrement=True),
+            Column('c2', String(30)),
+            Column('type', String(30))
+            )
+
+        Table('t2', metadata,
+            Column('c1', Integer, primary_key=True, test_needs_autoincrement=True),
+            Column('c2', String(30)),
+            Column('type', String(30)),
+            Column('t1.id', Integer, ForeignKey('t1.c1')))
+
+    def test_basic(self):
+        t2, t1 = self.tables.t2, self.tables.t1
+
+        class T(object):
+            pass
+
+        class SubT(T):
+            pass
+
+        class T2(object):
+            pass
+
+        class SubT2(T2):
+            pass
+
+        mapper(T, t1, polymorphic_on=t1.c.type, polymorphic_identity='t1')
+        mapper(SubT, None, inherits=T, polymorphic_identity='subt1', properties={
+            't2s': relationship(SubT2, lazy='subquery',
+                    backref=sa.orm.backref('subt', lazy='subquery'))
+        })
+        mapper(T2, t2, polymorphic_on=t2.c.type, polymorphic_identity='t2')
+        mapper(SubT2, None, inherits=T2, polymorphic_identity='subt2')
+
+        # testing a particular endless loop condition in eager load setup
+        create_session().query(SubT).all()
+
+class CyclicalInheritingEagerTestTwo(fixtures.DeclarativeMappedTest,
+                        testing.AssertsCompiledSQL):
+    __dialect__ = 'default'
+
+    @classmethod
+    def setup_classes(cls):
+        Base = cls.DeclarativeBasic
+        class PersistentObject(Base):
+            __tablename__ = 'persistent'
+            id = Column(Integer, primary_key=True, test_needs_autoincrement=True)
+
+        class Movie(PersistentObject):
+            __tablename__ = 'movie'
+            id = Column(Integer, ForeignKey('persistent.id'), primary_key=True)
+            director_id = Column(Integer, ForeignKey('director.id'))
+            title = Column(String(50))
+
+        class Director(PersistentObject):
+            __tablename__ = 'director'
+            id = Column(Integer, ForeignKey('persistent.id'), primary_key=True)
+            movies = relationship("Movie", foreign_keys=Movie.director_id)
+            name = Column(String(50))
+
+
+    def test_from_subclass(self):
+        Director = self.classes.Director
+
+        s = create_session()
+
+        ctx = s.query(Director).options(subqueryload('*'))._compile_context()
+
+        q = ctx.attributes[('subquery',
+                        (inspect(Director), inspect(Director).attrs.movies))]
+        self.assert_compile(q,
+            "SELECT anon_1.movie_id AS anon_1_movie_id, "
+            "anon_1.persistent_id AS anon_1_persistent_id, "
+            "anon_1.movie_director_id AS anon_1_movie_director_id, "
+            "anon_1.movie_title AS anon_1_movie_title, "
+            "anon_2.director_id AS anon_2_director_id FROM "
+            "(SELECT director.id AS director_id FROM persistent JOIN director "
+            "ON persistent.id = director.id) AS anon_2 "
+            "JOIN (SELECT persistent.id AS persistent_id, movie.id AS movie_id, "
+            "movie.director_id AS movie_director_id, "
+            "movie.title AS movie_title FROM persistent JOIN movie "
+            "ON persistent.id = movie.id) AS anon_1 "
+            "ON anon_2.director_id = anon_1.movie_director_id "
+            "ORDER BY anon_2.director_id")
+
+    def test_integrate(self):
+        Director = self.classes.Director
+        Movie = self.classes.Movie
+
+        session = Session(testing.db)
+        rscott = Director(name=u"Ridley Scott")
+        alien = Movie(title=u"Alien")
+        brunner = Movie(title=u"Blade Runner")
+        rscott.movies.append(brunner)
+        rscott.movies.append(alien)
+        session.add_all([rscott, alien, brunner])
+        session.commit()
+
+        session.close_all()
+        d = session.query(Director).options(subqueryload('*')).first()
+        assert len(list(session)) == 3
+
+
+class SubqueryloadDistinctTest(fixtures.DeclarativeMappedTest,
+                               testing.AssertsCompiledSQL):
+    __dialect__ = 'default'
+
+    run_inserts = 'once'
+    run_deletes = None
+
+    @classmethod
+    def setup_classes(cls):
+        Base = cls.DeclarativeBasic
+
+        class Director(Base):
+            __tablename__ = 'director'
+            id = Column(Integer, primary_key=True,
+                                    test_needs_autoincrement=True)
+            name = Column(String(50))
+
+        class DirectorPhoto(Base):
+            __tablename__ = 'director_photo'
+            id = Column(Integer, primary_key=True,
+                                    test_needs_autoincrement=True)
+            path = Column(String(255))
+            director_id = Column(Integer, ForeignKey('director.id'))
+            director = relationship(Director, backref="photos")
+
+        class Movie(Base):
+            __tablename__ = 'movie'
+            id = Column(Integer, primary_key=True, test_needs_autoincrement=True)
+            director_id = Column(Integer, ForeignKey('director.id'))
+            director = relationship(Director, backref="movies")
+            title = Column(String(50))
+            credits = relationship("Credit", backref="movie")
+
+        class Credit(Base):
+            __tablename__ = 'credit'
+            id = Column(Integer, primary_key=True, test_needs_autoincrement=True)
+            movie_id = Column(Integer, ForeignKey('movie.id'))
+
+    @classmethod
+    def insert_data(cls):
+        Movie = cls.classes.Movie
+        Director = cls.classes.Director
+        DirectorPhoto = cls.classes.DirectorPhoto
+        Credit = cls.classes.Credit
+
+        d = Director(name='Woody Allen')
+        d.photos = [DirectorPhoto(path='/1.jpg'),
+                    DirectorPhoto(path='/2.jpg')]
+        d.movies = [Movie(title='Manhattan', credits=[Credit(), Credit()]),
+                    Movie(title='Sweet and Lowdown', credits=[Credit()])]
+        sess = create_session()
+        sess.add_all([d])
+        sess.flush()
+
+    def test_distinct_strategy_opt_m2o(self):
+        self._run_test_m2o(True, None)
+        self._run_test_m2o(False, None)
+
+    def test_distinct_unrelated_opt_m2o(self):
+        self._run_test_m2o(None, True)
+        self._run_test_m2o(None, False)
+
+    def _run_test_m2o(self,
+            director_strategy_level,
+            photo_strategy_level):
+
+        # test where the innermost is m2o, e.g.
+        # Movie->director
+
+        Movie = self.classes.Movie
+        Director = self.classes.Director
+
+        Movie.director.property.distinct_target_key = director_strategy_level
+        Director.photos.property.distinct_target_key = photo_strategy_level
+
+        # the DISTINCT is controlled by
+        # only the Movie->director relationship, *not* the
+        # Director.photos
+        expect_distinct = director_strategy_level in (True, None)
+
+        s = create_session()
+
+        q = (
+            s.query(Movie)
+            .options(
+                subqueryload(Movie.director),
+                subqueryload(Movie.director, Director.photos)
+            )
+        )
+        ctx = q._compile_context()
+
+        q2 = ctx.attributes[
+            ('subquery', (inspect(Movie), inspect(Movie).attrs.director))
+        ]
+        self.assert_compile(
+            q2,
+            'SELECT director.id AS director_id, '
+            'director.name AS director_name, '
+            'anon_1.movie_director_id AS anon_1_movie_director_id '
+            'FROM (SELECT%s movie.director_id AS movie_director_id '
+            'FROM movie) AS anon_1 '
+            'JOIN director ON director.id = anon_1.movie_director_id '
+            'ORDER BY anon_1.movie_director_id' % (
+                    " DISTINCT" if expect_distinct else "")
+        )
+
+        ctx2 = q2._compile_context()
+        result = s.execute(q2)
+        rows = result.fetchall()
+
+        if expect_distinct:
+            eq_(rows, [
+                (1, 'Woody Allen', 1),
+            ])
+        else:
+            eq_(rows, [
+                (1, 'Woody Allen', 1), (1, 'Woody Allen', 1),
+            ])
+
+        q3 = ctx2.attributes[
+            ('subquery', (inspect(Director), inspect(Director).attrs.photos))
+        ]
+
+        self.assert_compile(
+            q3,
+            'SELECT director_photo.id AS director_photo_id, '
+            'director_photo.path AS director_photo_path, '
+            'director_photo.director_id AS director_photo_director_id, '
+            'director_1.id AS director_1_id '
+            'FROM (SELECT%s movie.director_id AS movie_director_id '
+            'FROM movie) AS anon_1 '
+            'JOIN director AS director_1 ON director_1.id = anon_1.movie_director_id '
+            'JOIN director_photo ON director_1.id = director_photo.director_id '
+            'ORDER BY director_1.id' % (
+                    " DISTINCT" if expect_distinct else "")
+        )
+        result = s.execute(q3)
+        rows = result.fetchall()
+        if expect_distinct:
+            eq_(set(tuple(r) for r in rows), set([
+                (1, u'/1.jpg', 1, 1),
+                (2, u'/2.jpg', 1, 1),
+            ]))
+        else:
+            eq_(set(tuple(r) for r in rows), set([
+                (1, u'/1.jpg', 1, 1),
+                (2, u'/2.jpg', 1, 1),
+                (1, u'/1.jpg', 1, 1),
+                (2, u'/2.jpg', 1, 1),
+            ]))
+
+
+        movies = q.all()
+
+        # check number of persistent objects in session
+        eq_(len(list(s)), 5)
+
+    def test_cant_do_distinct_in_joins(self):
+        """the DISTINCT feature here works when the m2o is in the innermost
+        mapper, but when we are just joining along relationships outside
+        of that, we can still have dupes, and there's no solution to that.
+
+        """
+        Movie = self.classes.Movie
+        Credit = self.classes.Credit
+
+        Credit.movie.property.distinct_target_key = False
+        Movie.director.property.distinct_target_key = False
+
+        s = create_session()
+
+        q = (
+            s.query(Credit)
+            .options(
+                subqueryload(Credit.movie),
+                subqueryload(Credit.movie, Movie.director)
+            )
+        )
+
+        ctx = q._compile_context()
+
+        q2 = ctx.attributes[
+            ('subquery', (inspect(Credit), Credit.movie.property))
+        ]
+        ctx2 = q2._compile_context()
+        q3 = ctx2.attributes[
+            ('subquery', (inspect(Movie), Movie.director.property))
+        ]
+
+        # three rows due to dupe at Credit.movie level
+        # as well as Movie.director level
+        result = s.execute(q3)
+        eq_(
+            result.fetchall(),
+            [
+                (1, 'Woody Allen', 1), (1, 'Woody Allen', 1),
+                    (1, 'Woody Allen', 1)
+            ]
+        )
